@@ -10,18 +10,37 @@
 #include "evernoteimportdialog.h"
 #include "ui_evernoteimportdialog.h"
 #include "filedialog.h"
+#include <QRegularExpression>
+#include <QRegularExpressionMatch>
+#include <QRegularExpressionMatchIterator>
+#include <QtCore/QSettings>
 
 EvernoteImportDialog::EvernoteImportDialog(QWidget *parent) :
         MasterDialog(parent),
     ui(new Ui::EvernoteImportDialog) {
     ui->setupUi(this);
+    setupMetaDataTreeWidgetItems();
     ui->progressBar->hide();
     ui->progressBar->setMinimum(0);
     ui->progressBar->setValue(0);
     _importCount = 0;
+
+    QSettings settings;
+    ui->imageImportCheckBox->setChecked(settings.value(
+            "EvernoteImport/ImageImportCheckBoxChecked", true).toBool());
+    ui->attachmentImportCheckBox->setChecked(settings.value(
+            "EvernoteImport/AttachmentImportCheckBoxChecked", true).toBool());
 }
 
 EvernoteImportDialog::~EvernoteImportDialog() {
+    QSettings settings;
+    settings.setValue("EvernoteImport/ImageImportCheckBoxChecked",
+            ui->imageImportCheckBox->isChecked());
+    settings.setValue("EvernoteImport/AttachmentImportCheckBoxChecked",
+            ui->attachmentImportCheckBox->isChecked());
+
+    storeMetaDataTreeWidgetItemsCheckedState();
+
     delete ui;
 }
 
@@ -45,21 +64,6 @@ void EvernoteImportDialog::on_fileButton_clicked() {
 
         if (!fileName.isEmpty()) {
             ui->fileLineEdit->setText(fileName);
-
-            QFile file(fileName);
-
-            if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
-                qCritical() << file.errorString() <<
-                                          " (path: " << fileName << ")";
-                return;
-            }
-
-            QString data = file.readAll();
-
-            QCoreApplication::processEvents();
-            initNoteCount(data);
-            QCoreApplication::processEvents();
-            importNotes(data);
         }
     }
 }
@@ -404,22 +408,7 @@ QString EvernoteImportDialog::getMarkdownForMediaFileData(
     QString data = mediaFileData.data;
     QString imageSuffix = mediaFileData.suffix;
 
-    // create a temporary file for the image
-    QTemporaryFile *tempFile = new QTemporaryFile(
-            QDir::tempPath() + QDir::separator() + "media-XXXXXX." +
-            imageSuffix);
-
-    if (!tempFile->open()) {
-        return "";
-    }
-
-    // write image to the temporary file
-    tempFile->write(QByteArray::fromBase64(data.toLatin1()));
-
-    // store the temporary image in the media folder and return the
-    // markdown code
-    QString markdownCode = Note::getInsertMediaMarkdown(tempFile);
-    return markdownCode;
+    return Utils::Misc::importMediaFromBase64(data, imageSuffix);
 }
 
 /**
@@ -462,6 +451,8 @@ void EvernoteImportDialog::importNotes(QString data) {
     query.setFocus(data);
     query.setQuery("en-export/note");
 
+    bool importMetaData = isMetaDataChecked();
+
     QXmlResultItems result;
     if (query.isValid()) {
         query.evaluateTo(&result);
@@ -491,6 +482,19 @@ void EvernoteImportDialog::importNotes(QString data) {
 //            content.replace(QRegularExpression("<ul.*?>"), "\n<ul>");
 //            content.replace(QRegularExpression("<ol.*?>"), "\n<ol>");
 
+            // remove web-clip code
+//            content.remove("<div style=\"-evernote-webclip:true;\">");
+
+            // replace code blocks
+            content.replace(QRegularExpression(
+                    R"(<div style="box-sizing.+?-en-codeblock:\s*true;"><div>(.+?)<\/div><\/div>)",
+                    QRegularExpression::MultilineOption),
+                            "\n```\n\\1\n```\n");
+            content.replace(QRegularExpression(
+                    R"(<div style="-en-codeblock:\s*true;.+?"><div>(.+?)<\/div><\/div>)",
+                    QRegularExpression::MultilineOption),
+                            "\n```\n\\1\n```\n");
+
             // add a linebreak instead of div-containers
             content.replace(QRegularExpression("<\\/div>"), "\n");
 
@@ -527,11 +531,22 @@ void EvernoteImportDialog::importNotes(QString data) {
             title.remove("\"");
 #endif
 
-            QString noteText = Note::createNoteHeader(title) + content;
+            QString noteText = Note::createNoteHeader(title);
+
+            if (importMetaData) {
+                noteText += generateMetaDataMarkdown(query);
+            }
+
+            noteText += content;
 
             Note note = Note();
 //            note.setName(title);
             note.setNoteText(noteText);
+
+            // in case the user enabled that the filename can be different
+            // from the note name
+            note.handleNoteTextFileName();
+
             note.setNoteSubFolderId(noteSubFolder.getId());
             note.store();
             note.storeNoteTextFileToDisk();
@@ -582,3 +597,188 @@ void EvernoteImportDialog::tagNote(QXmlQuery &query, Note &note) {
         }
     }
 }
+
+/**
+ * Adds a metadata tree widget item
+ *
+ * @param name
+ * @param attributeName
+ * @param parentItem
+ * @return
+ */
+QTreeWidgetItem *EvernoteImportDialog::addMetaDataTreeWidgetItem(
+        QString name,
+        QString attributeName,
+        QTreeWidgetItem *parentItem) {
+    auto *item = new QTreeWidgetItem();
+    item->setText(0, name);
+
+    if (parentItem == nullptr) {
+        ui->metaDataTreeWidget->addTopLevelItem(item);
+        item->setExpanded(true);
+    } else {
+        item->setToolTip(0, attributeName);
+        item->setData(0, Qt::UserRole, attributeName);
+        item->setFlags(item->flags() | Qt::ItemIsUserCheckable);
+
+        QSettings settings;
+        auto metaDataUnCheckedList = settings.value(
+                "EvernoteImport/MetaDataUnCheckedList").toStringList();
+        item->setCheckState(0,
+                metaDataUnCheckedList.contains(attributeName) ?
+                Qt::Unchecked : Qt::Checked);
+
+        parentItem->addChild(item);
+    }
+
+    return item;
+}
+
+/**
+ * Setup the metadata tree widget items
+ */
+void EvernoteImportDialog::setupMetaDataTreeWidgetItems() {
+    auto *basicAttributesItem = addMetaDataTreeWidgetItem(
+            tr("Basic attributes"));
+    addMetaDataTreeWidgetItem(tr("Created date"), "created",
+            basicAttributesItem);
+    addMetaDataTreeWidgetItem(tr("Updated date"), "updated",
+            basicAttributesItem);
+
+    auto *noteAttributesItem = addMetaDataTreeWidgetItem(tr("Note attributes"));
+    addMetaDataTreeWidgetItem(tr("Subject date"),
+            "note-attributes/subject-date", noteAttributesItem);
+    addMetaDataTreeWidgetItem(tr("Latitude"),
+            "note-attributes/latitude", noteAttributesItem);
+    addMetaDataTreeWidgetItem(tr("Longitude"),
+            "note-attributes/longitude", noteAttributesItem);
+    addMetaDataTreeWidgetItem(tr("Altitude"),
+            "note-attributes/altitude", noteAttributesItem);
+    addMetaDataTreeWidgetItem(tr("Author"),
+            "note-attributes/author", noteAttributesItem);
+    addMetaDataTreeWidgetItem(tr("Source"),
+            "note-attributes/source", noteAttributesItem);
+    addMetaDataTreeWidgetItem(tr("Source URL"),
+            "note-attributes/source-url", noteAttributesItem);
+    addMetaDataTreeWidgetItem(tr("Source application"),
+            "note-attributes/source-application", noteAttributesItem);
+    addMetaDataTreeWidgetItem(tr("Reminder order"),
+            "note-attributes/reminder-order", noteAttributesItem);
+    addMetaDataTreeWidgetItem(tr("Reminder time"),
+            "note-attributes/reminder-time", noteAttributesItem);
+    addMetaDataTreeWidgetItem(tr("Reminder done time"),
+            "note-attributes/reminder-done-time", noteAttributesItem);
+    addMetaDataTreeWidgetItem(tr("Place name"),
+            "note-attributes/place-name", noteAttributesItem);
+    addMetaDataTreeWidgetItem(tr("Content class"),
+            "note-attributes/content-class", noteAttributesItem);
+    addMetaDataTreeWidgetItem(tr("Application data"),
+            "note-attributes/application-data", noteAttributesItem);
+}
+
+/**
+ * Setup the metadata tree widget items
+ */
+void EvernoteImportDialog::storeMetaDataTreeWidgetItemsCheckedState() {
+    QList<QTreeWidgetItem *> items = ui->metaDataTreeWidget->findItems(
+            QString("*"), Qt::MatchWrap | Qt::MatchWildcard |
+                          Qt::MatchRecursive);
+    QSettings settings;
+    QStringList metaDataUnCheckedList;
+
+    Q_FOREACH(QTreeWidgetItem *item, items) {
+            QString attributeName = item->data(0, Qt::UserRole).toString();
+
+            if (item->checkState(0) != Qt::Checked) {
+                metaDataUnCheckedList << attributeName;
+            }
+        }
+
+    settings.setValue("EvernoteImport/MetaDataUnCheckedList",
+            metaDataUnCheckedList);
+}
+
+/**
+ * Checks if a metadata attribute is checked
+ */
+bool EvernoteImportDialog::isMetaDataChecked() {
+    QList<QTreeWidgetItem *> items = ui->metaDataTreeWidget->findItems(
+            QString("*"), Qt::MatchWrap | Qt::MatchWildcard |
+                          Qt::MatchRecursive);
+
+    Q_FOREACH(QTreeWidgetItem *item, items) {
+            if (item->checkState(0) == Qt::Checked) {
+                return true;
+            }
+        }
+
+    return false;
+}
+
+/**
+ * Generates the metadata markdown table for a note
+ */
+QString EvernoteImportDialog::generateMetaDataMarkdown(QXmlQuery query) {
+    QString resultText;
+    QString tableText;
+    QList<QTreeWidgetItem *> items = ui->metaDataTreeWidget->findItems(
+            QString("*"), Qt::MatchWrap | Qt::MatchWildcard |
+                          Qt::MatchRecursive);
+
+    Q_FOREACH(QTreeWidgetItem *item, items) {
+            if (item->checkState(0) != Qt::Checked) {
+                continue;
+            }
+
+            QString name = item->text(0);
+            QString attributeName = item->data(0, Qt::UserRole).toString();
+
+            QString attribute;
+            query.setQuery(attributeName + "/text()");
+            query.evaluateTo(&attribute);
+            attribute = attribute.trimmed();
+
+            if (attribute.isEmpty()) {
+                continue;
+            }
+
+            tableText += "| " + name + " | " + attribute + " |\n";
+        }
+
+    if (!tableText.isEmpty()) {
+        resultText = "| " + tr("Attribute") + " | " + tr("Value") + " |\n"
+                     "|---|---|\n" +
+                     tableText;
+    }
+
+    return resultText;
+}
+
+/**
+ * Reads the enex files and imports the notes
+ */
+void EvernoteImportDialog::on_importButton_clicked() {
+    auto fileName = ui->fileLineEdit->text();
+
+    if (fileName.isEmpty()) {
+        return;
+    }
+
+    QFile file(fileName);
+
+    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
+        qCritical() << file.errorString() << " (path: " << fileName << ")";
+        return;
+    }
+
+    ui->importButton->setEnabled(false);
+
+    QString data = file.readAll();
+
+    QCoreApplication::processEvents();
+    initNoteCount(data);
+    QCoreApplication::processEvents();
+    importNotes(data);
+
+    ui->importButton->setEnabled(true);
+}
