@@ -7,8 +7,11 @@
 #include <QNetworkRequest>
 #include <QNetworkReply>
 #include <QFileDialog>
+#include <QSettings>
 #include "entities/note.h"
 #include "helpers/htmlentities.h"
+#include <QRegularExpression>
+#include <QRegularExpressionMatch>
 
 LinkDialog::LinkDialog(QString dialogTitle, QWidget *parent) :
         MasterDialog(parent),
@@ -73,6 +76,14 @@ QString LinkDialog::getURL() {
     return url;
 }
 
+QString LinkDialog::getLinkName() {
+    return ui->nameLineEdit->text().trimmed();
+}
+
+QString LinkDialog::getLinkDescription() {
+    return ui->descriptionLineEdit->text().trimmed();
+}
+
 //
 // Event filters on the NoteSearchDialog
 //
@@ -187,11 +198,19 @@ QString LinkDialog::getTitleForUrl(QUrl url) {
  * Selects a local file to link to
  */
 void LinkDialog::on_fileUrlButton_clicked() {
-    QUrl fileUrl = QFileDialog::getOpenFileUrl(this,
-                                               tr("Select file to link to"));
-    QString fileUrlString = fileUrl.toString();
+    QSettings settings;
+    // load last url
+    QUrl fileUrl = settings.value("LinkDialog/lastSelectedFileUrl").toUrl();
 
-    if (fileUrlString != "") {
+    fileUrl = QFileDialog::getOpenFileUrl(this, tr("Select file to link to"),
+                                          fileUrl);
+    QString fileUrlString = fileUrl.toString(QUrl::FullyEncoded);
+
+    if (!fileUrlString.isEmpty()) {
+        // store url for the next time
+        settings.setValue("LinkDialog/lastSelectedFileUrl", fileUrlString);
+
+        // write the file-url to the url text-edit
         ui->urlEdit->setText(fileUrlString);
     }
 }
