@@ -1,4 +1,6 @@
 #include <services/owncloudservice.h>
+#include <utils/gui.h>
+#include <utils/misc.h>
 #include "sharedialog.h"
 #include "ui_sharedialog.h"
 
@@ -31,13 +33,15 @@ void ShareDialog::updateDialog() {
     ui->linkCheckBox->setChecked(note.isShared());
     ui->linkUrlLineEdit->setVisible(note.isShared());
     ui->linkUrlLineEdit->setText(note.getShareUrl());
+    ui->infoLabel1->setText(Utils::Misc::replaceOwnCloudText(
+            ui->infoLabel1->text()));
 }
 
 /**
  * Shares or removes the share from the current note
  */
 void ShareDialog::on_linkCheckBox_toggled(bool checked) {
-    OwnCloudService *ownCloud = new OwnCloudService(this);
+    auto *ownCloud = new OwnCloudService(this);
 
     const QSignalBlocker blocker(ui->linkCheckBox);
     Q_UNUSED(blocker);
@@ -48,6 +52,14 @@ void ShareDialog::on_linkCheckBox_toggled(bool checked) {
     if (checked) {
         // share the note file
         ownCloud->shareNote(note, this);
+
+        Utils::Gui::information(
+                this, "",
+                Utils::Misc::replaceOwnCloudText(
+                        tr("Keep in mind that you still have to sync your "
+                           "notes with your server by using the ownCloud "
+                           "desktop sync tool to be able to share notes with "
+                           "others!")), "share-sync-information");
     } else {
         // remove the share
         ownCloud->removeNoteShare(note, this);
