@@ -7,6 +7,8 @@
 #include <QDir>
 #include <QSqlError>
 #include <QRegularExpression>
+#include <QRegularExpressionMatch>
+#include <QRegularExpressionMatchIterator>
 #include <QUrl>
 #include <QCryptographicHash>
 #include "libraries/simplecrypt/simplecrypt.h"
@@ -15,6 +17,7 @@
 #include "libraries/botan/botan.h"
 #include "tag.h"
 #include "notesubfolder.h"
+#include "trashitem.h"
 #include <utils/misc.h>
 #include <services/scriptingservice.h>
 #include <QMimeDatabase>
@@ -86,6 +89,16 @@ int Note::getNoteSubFolderId() {
     return this->noteSubFolderId;
 }
 
+bool Note::isInCurrentNoteSubFolder() {
+    const int currentNoteSubFolderId = NoteSubFolder::activeNoteSubFolderId();
+
+    if (currentNoteSubFolderId <= 0) {
+        return true;
+    }
+
+    return this->noteSubFolderId == currentNoteSubFolderId;
+}
+
 void Note::setNoteSubFolder(NoteSubFolder noteSubFolder) {
     setNoteSubFolderId(noteSubFolder.getId());
 }
@@ -163,6 +176,30 @@ Note Note::fetch(int id) {
     return note;
 }
 
+/**
+ * Fetches a note by note name with a regular expression
+ *
+ * @param regExp
+ * @param noteSubFolderId if not set all notes will be searched
+ * @return
+ */
+Note Note::fetchByName(QRegularExpression regExp, int noteSubFolderId) {
+    QList<Note> noteList = noteSubFolderId == -1 ?
+            fetchAll() :
+            fetchAllByNoteSubFolderId(noteSubFolderId);
+
+    // since there is no regular expression search in Qt's sqlite
+    // implementation we have to iterate
+    Q_FOREACH(Note note, noteList) {
+            QRegularExpressionMatch match = regExp.match(note.getName());
+            if (match.hasMatch()) {
+                return note;
+            }
+    }
+
+    return Note();
+}
+
 Note Note::fetchByFileName(QString fileName, int noteSubFolderId) {
     Note note;
 
@@ -249,7 +286,36 @@ bool Note::copyToPath(QString destinationPath) {
             qDebug() << "New file name:" << destinationFileName;
         }
 
-        return file.copy(destinationFileName);
+        // copy the note file to the destination
+        bool isFileCopied = file.copy(destinationFileName);
+
+        if (isFileCopied) {
+            QStringList mediaFileList = getMediaFileList();
+
+            if (mediaFileList.count() > 0) {
+                QDir mediaDir(destinationPath + QDir::separator() + "media");
+
+                // created the media folder if it doesn't exist
+                if (!mediaDir.exists()) {
+                    mediaDir.mkpath(mediaDir.path());
+                }
+
+                if (mediaDir.exists()) {
+                    // copy all images to the media folder inside destinationPath
+                    Q_FOREACH(QString fileName, mediaFileList) {
+                            QFile mediaFile(NoteFolder::currentMediaPath() +
+                                            QDir::separator() + fileName);
+
+                            if (mediaFile.exists()) {
+                                mediaFile.copy(mediaDir.path() +
+                                               QDir::separator() + fileName);
+                            }
+                        }
+                }
+            }
+        }
+
+        return isFileCopied;
     }
 
     return false;
@@ -271,6 +337,52 @@ bool Note::moveToPath(QString destinationPath) {
     return false;
 }
 
+/**
+ * Returns a list of all linked media file of the current note
+ * @return
+ */
+QStringList Note::getMediaFileList() {
+    QString text = getNoteText();
+    QStringList fileList;
+
+    // match image links like ![media-qV920](file://media/608766373.gif)
+    QRegularExpression re(
+            "!\\[.*?\\]\\(file:\\/\\/media/(.+?)\\)");
+    QRegularExpressionMatchIterator i = re.globalMatch(text);
+
+    // remove all found images from the orphaned files list
+    while (i.hasNext()) {
+        QRegularExpressionMatch match = i.next();
+        QString fileName = match.captured(1);
+        fileList << fileName;
+    }
+
+    return fileList;
+}
+
+/**
+ * Returns a list of all linked attachments of the current note
+ * @return
+ */
+QStringList Note::getAttachmentsFileList() {
+    QString text = getNoteText();
+    QStringList fileList;
+
+    // match attachment links like [956321614](file://attachments/956321614.pdf)
+    QRegularExpression re(
+            "\\[.*?\\]\\(file:\\/\\/attachments/(.+?)\\)");
+    QRegularExpressionMatchIterator i = re.globalMatch(text);
+
+    // remove all found images from the orphaned files list
+    while (i.hasNext()) {
+        QRegularExpressionMatch match = i.next();
+        QString fileName = match.captured(1);
+        fileList << fileName;
+    }
+
+    return fileList;
+}
+
 /**
  * Fetches a note by its share id
  *
@@ -528,7 +640,8 @@ QList<Note> Note::search(QString text) {
     return noteList;
 }
 
-QList<QString> Note::searchAsNameList(QString text, bool searchInNameOnly) {
+QList<QString> Note::searchAsNameListInCurrentNoteSubFolder(
+        QString text, bool searchInNameOnly) {
     QSqlDatabase db = QSqlDatabase::database("memory");
     QSqlQuery query(db);
 
@@ -553,6 +666,28 @@ QList<QString> Note::searchAsNameList(QString text, bool searchInNameOnly) {
     return nameList;
 }
 
+QList<QString> Note::searchAsNameList(QString text, bool searchInNameOnly) {
+    QSqlDatabase db = QSqlDatabase::database("memory");
+    QSqlQuery query(db);
+
+    QList<QString> nameList;
+    QString textSearchSql = !searchInNameOnly ? "OR note_text LIKE :text " : "";
+
+    query.prepare("SELECT name FROM note WHERE (name LIKE :text " +
+            textSearchSql + ") ORDER BY file_last_modified DESC");
+    query.bindValue(":text", "%" + text + "%");
+
+    if (!query.exec()) {
+        qWarning() << __func__ << ": " << query.lastError();
+    } else {
+        for (int r = 0; query.next(); r++) {
+            nameList.append(query.value("name").toString());
+        }
+    }
+
+    return nameList;
+}
+
 /**
  * Searches for text in notes and returns the note ids
  *
@@ -663,7 +798,7 @@ QStringList Note::buildQueryStringList(QString searchString,
     return queryStrings;
 }
 
-QStringList Note::fetchNoteNames() {
+QStringList Note::fetchNoteNamesInCurrentNoteSubFolder() {
     QSqlDatabase db = QSqlDatabase::database("memory");
     QSqlQuery query(db);
 
@@ -689,6 +824,29 @@ QStringList Note::fetchNoteNames() {
     return list;
 }
 
+QStringList Note::fetchNoteNames() {
+    QSqlDatabase db = QSqlDatabase::database("memory");
+    QSqlQuery query(db);
+
+    QStringList list;
+
+    query.prepare("SELECT DISTINCT(name) FROM note "
+                  "ORDER BY file_last_modified DESC");
+
+    if (!query.exec()) {
+        qWarning() << __func__ << ": " << query.lastError();
+    } else {
+        for (int r = 0; query.next(); r++) {
+            QString name = query.value("name").toString();
+            if (!name.isEmpty()) {
+                list.append(name);
+            }
+        }
+    }
+
+    return list;
+}
+
 QStringList Note::fetchNoteFileNames() {
     QSqlDatabase db = QSqlDatabase::database("memory");
     QSqlQuery query(db);
@@ -708,7 +866,32 @@ QStringList Note::fetchNoteFileNames() {
     return list;
 }
 
+QList<int> Note::fetchAllIdsByNoteTextPart(QString textPart) {
+    QSqlDatabase db = QSqlDatabase::database("memory");
+    QSqlQuery query(db);
+
+    QList<int> list;
+
+    query.prepare("SELECT id FROM note WHERE note_text LIKE :text "
+                          "ORDER BY file_last_modified DESC");
+    query.bindValue(":text", "%" + textPart + "%");
+
+    if (!query.exec()) {
+        qWarning() << __func__ << ": " << query.lastError();
+    } else {
+        for (int r = 0; query.next(); r++) {
+            list.append(query.value("id").toInt());
+        }
+    }
+
+    return list;
+}
+
 bool Note::storeNewText(QString text) {
+    if (!this->fileWriteable()) {
+        return false;
+    }
+
     this->noteText = text;
     this->hasDirtyData = true;
 
@@ -845,6 +1028,7 @@ bool Note::store() {
 bool Note::storeNoteTextFileToDisk() {
     QString oldName = name;
     QString oldNoteFilePath = fullNoteFilePath();
+    TrashItem trashItem = TrashItem::prepare(this);
 
     if (allowDifferentFileName()) {
         // check if a QML function wants to set an other note file name and
@@ -878,6 +1062,13 @@ bool Note::storeNoteTextFileToDisk() {
 
     // assign the tags to the new name if the name has changed
     if (oldName != newName) {
+        if (TrashItem::isLocalTrashEnabled()) {
+            qDebug() << __func__ << " - 'trashItem': " << trashItem;
+
+            // trash the old note
+            trashItem.doTrashing();
+        }
+
         // TODO(pbek): we need to heed note subfolders here
         Tag::renameNoteFileNamesOfLinks(oldName, newName);
 
@@ -969,7 +1160,7 @@ QString Note::cleanupFileName(QString name) {
 QString Note::extendedCleanupFileName(QString name) {
     // replace characters that cause problems on certain filesystems when
     // present in filenames with underscores
-    name.replace(QRegularExpression("[\\/\\\\:<>\\\"\\|\\?\\*]"), "_");
+    name.replace(QRegularExpression("[\\/\\\\:<>\\\"\\|\\?\\*]"), " ");
 
     return name;
 }
@@ -1104,6 +1295,10 @@ bool Note::canWriteToNoteFile() {
 }
 
 bool Note::updateNoteTextFromDisk() {
+    if (!isFetched()) {
+        return false;
+    }
+
     QFile file(fullNoteFilePath());
 
     if (!file.open(QIODevice::ReadOnly)) {
@@ -1276,7 +1471,8 @@ int Note::storeDirtyNotesToDisk(Note &currentNote, bool *currentNoteChanged,
     }
 }
 
-void Note::createFromFile(QFile &file, int noteSubFolderId) {
+void Note::createFromFile(QFile &file, int noteSubFolderId,
+                          bool withNoteNameHook) {
     if (file.open(QIODevice::ReadOnly)) {
         QTextStream in(&file);
         in.setCodec("UTF-8");
@@ -1302,6 +1498,17 @@ void Note::createFromFile(QFile &file, int noteSubFolderId) {
         this->fileCreated = fileInfo.created();
         this->fileLastModified = fileInfo.lastModified();
         this->store();
+
+        if (withNoteNameHook) {
+            // check if a name was set in a script
+            QString hookName = ScriptingService::instance()->
+                    callHandleNoteNameHook(this);
+
+            if (!hookName.isEmpty()) {
+                this->name = hookName;
+                this->store();
+            }
+        }
     }
 }
 
@@ -1312,7 +1519,8 @@ void Note::createFromFile(QFile &file, int noteSubFolderId) {
  * @param noteSubFolder
  * @return
  */
-Note Note::updateOrCreateFromFile(QFile &file, NoteSubFolder noteSubFolder) {
+Note Note::updateOrCreateFromFile(QFile &file, NoteSubFolder noteSubFolder,
+                                  bool withNoteNameHook) {
     QFileInfo fileInfo(file);
     Note note = fetchByFileName(fileInfo.fileName(), noteSubFolder.getId());
 
@@ -1322,7 +1530,7 @@ Note Note::updateOrCreateFromFile(QFile &file, NoteSubFolder noteSubFolder) {
     if ((fileInfo.size() != note.getFileSize()) ||
                 (fileInfo.lastModified() > note.getModified())) {
         // load file data and store note
-        note.createFromFile(file, noteSubFolder.getId());
+        note.createFromFile(file, noteSubFolder.getId(), withNoteNameHook);
 
 //        qDebug() << __func__ << " - 'file modified': " << file.fileName();
     }
@@ -1429,6 +1637,12 @@ bool Note::renameNoteFile(QString newName) {
         return false;
     }
 
+    if (TrashItem::isLocalTrashEnabled()) {
+        // add note to trash
+        bool trashResult = TrashItem::add(this);
+        qDebug() << __func__ << " - 'trashResult': " << trashResult;
+    }
+
     // get the note file to rename it
     QFile file(fullNoteFilePath());
 
@@ -1441,11 +1655,19 @@ bool Note::renameNoteFile(QString newName) {
     return file.rename(fullNoteFilePath());
 }
 
-//
-// remove the file of the note
-//
+/**
+ * Removes the file of the note
+ *
+ * @return
+ */
 bool Note::removeNoteFile() {
     if (this->fileExists()) {
+        if (TrashItem::isLocalTrashEnabled()) {
+            // add note to trash
+            bool trashResult = TrashItem::add(this);
+            qDebug() << __func__ << " - 'trashResult': " << trashResult;
+        }
+
         QFile file(fullNoteFilePath());
         qDebug() << __func__ << " - 'this->fileName': " << this->fileName;
         qDebug() << __func__ << " - 'file': " << file.fileName();
@@ -1479,6 +1701,29 @@ QString Note::toMarkdownHtml(QString notesPath, int maxImageWidth,
         return _noteTextHtml;
     }
 
+    QString result = textToMarkdownHtml(str, notesPath, maxImageWidth,
+                                        forExport, base64Images);
+
+    // cache the html output and conversion hash
+    _noteTextHtmlConversionHash = hash;
+    _noteTextHtml = result;
+
+    return result;
+}
+
+/**
+ * Converts a markdown string for a note to html
+ *
+ * @param str
+ * @param notesPath
+ * @param maxImageWidth
+ * @param forExport
+ * @param base64Images
+ * @return
+ */
+QString Note::textToMarkdownHtml(QString str, QString notesPath,
+                                 int maxImageWidth,
+                                 bool forExport, bool base64Images) {
     hoedown_renderer *renderer =
             hoedown_html_renderer_new(HOEDOWN_HTML_USE_XHTML, 32);
 
@@ -1487,7 +1732,7 @@ QString Note::toMarkdownHtml(QString notesPath, int maxImageWidth,
     // HOEDOWN_EXT_MATH and HOEDOWN_EXT_MATH_EXPLICIT don't seem to do anything
     hoedown_extensions extensions =
             (hoedown_extensions) ((HOEDOWN_EXT_BLOCK | HOEDOWN_EXT_SPAN |
-                    HOEDOWN_EXT_MATH_EXPLICIT) & ~HOEDOWN_EXT_QUOTE);
+                                   HOEDOWN_EXT_MATH_EXPLICIT) & ~HOEDOWN_EXT_QUOTE);
     hoedown_document *document = hoedown_document_new(renderer, extensions, 32);
 
     QString windowsSlash = "";
@@ -1576,7 +1821,7 @@ QString Note::toMarkdownHtml(QString notesPath, int maxImageWidth,
     hoedown_document_free(document);
     hoedown_html_renderer_free(renderer);
 
-    QSettings(settings);
+    QSettings settings;
     QString fontString = settings.value("MainWindow/noteTextView.code.font")
             .toString();
 
@@ -1594,17 +1839,15 @@ QString Note::toMarkdownHtml(QString notesPath, int maxImageWidth,
 
     bool darkModeColors = settings.value("darkModeColors").toBool();
 
+    QString codeForegroundColor = darkModeColors ? "#ffffff" : "#000000";
     QString codeBackgroundColor = darkModeColors ? "#444444" : "#f1f1f1";
 
     // do some more code formatting
     codeStyleSheet += QString(
-            "pre, code { padding: 16px; overflow: auto;"
+            "code { padding: 16px; overflow: auto;"
                     " line-height: 1.45em; background-color: %1;"
-                    " border-radius: 3px; }").arg(codeBackgroundColor);
-
-    // remove double code blocks
-    result.replace("<pre><code>", "<pre>")
-            .replace("</code></pre>", "</pre>");
+                    " border-radius: 3px; color: %2; }").arg(
+            codeBackgroundColor, codeForegroundColor);
 
     // correct the strikeout tag
     result.replace(QRegularExpression("<del>([^<]+)<\\/del>"), "<s>\\1</s>");
@@ -1627,20 +1870,24 @@ QString Note::toMarkdownHtml(QString notesPath, int maxImageWidth,
         }
 
         result = QString("<html><head><meta charset=\"utf-8\"/><style>"
-                    "h1 { margin: 5px 0 20px 0; }"
-                    "h2, h3 { margin: 10px 0 15px 0; }"
-                    "img { max-width: 100%; }"
-                    "a { color: #FF9137; text-decoration: none; } %1 %2 %4"
-                    "</style></head><body>%3</body></html>")
-            .arg(codeStyleSheet, exportStyleSheet, result, rtlStyle);
+                                 "h1 { margin: 5px 0 20px 0; }"
+                                 "h2, h3 { margin: 10px 0 15px 0; }"
+                                 "img { max-width: 100%; }"
+                                 "table {border-spacing: 0; border-style: solid; border-width: 1px; border-collapse: collapse; margin-top: 0.5em;}"
+                                 "th, td {padding: 2px 5px;}"
+                                 "a { color: #FF9137; text-decoration: none; } %1 %2 %4"
+                                 "</style></head><body>%3</body></html>")
+                .arg(codeStyleSheet, exportStyleSheet, result, rtlStyle);
     } else {
         // for preview
         result = QString("<html><head><style>"
-                    "h1 { margin: 5px 0 20px 0; }"
-                    "h2, h3 { margin: 10px 0 15px 0; }"
-                    "a { color: #FF9137; text-decoration: none; } %1 %3"
-                    "</style></head><body>%2</body></html>")
-            .arg(codeStyleSheet, result, rtlStyle);
+                                 "h1 { margin: 5px 0 20px 0; }"
+                                 "h2, h3 { margin: 10px 0 15px 0; }"
+                                 "table {border-spacing: 0; border-style: solid; border-width: 1px; border-collapse: collapse; margin-top: 0.5em;}"
+                                 "th, td {padding: 2px 5px;}"
+                                 "a { color: #FF9137; text-decoration: none; } %1 %3"
+                                 "</style></head><body>%2</body></html>")
+                .arg(codeStyleSheet, result, rtlStyle);
     }
 
     // check if width of embedded local images is too high
@@ -1660,14 +1907,14 @@ QString Note::toMarkdownHtml(QString notesPath, int maxImageWidth,
                                                                   fileName) +
                                        "\""),
                     QString("<img src=\"file://%2\"").arg(windowsSlash +
-                                                                  fileName));
+                                                          fileName));
         } else {
             // for preview
             // cap the image width at maxImageWidth (note text view width)
             int originalWidth = image.width();
             int displayWidth = (originalWidth > maxImageWidth)
-                    ? maxImageWidth
-                    : originalWidth;
+                               ? maxImageWidth
+                               : originalWidth;
 
             result.replace(
                     QRegularExpression("<img src=\"file:\\/\\/" +
@@ -1676,7 +1923,7 @@ QString Note::toMarkdownHtml(QString notesPath, int maxImageWidth,
                                        "\""),
                     QString("<img width=\"%1\" src=\"file://%2\"").arg(
                             QString::number(displayWidth), windowsSlash +
-                            fileName));
+                                                           fileName));
         }
 
         // encode the image base64
@@ -1712,11 +1959,6 @@ QString Note::toMarkdownHtml(QString notesPath, int maxImageWidth,
     }
 
 //    qDebug() << __func__ << " - 'result': " << result;
-
-    // cache the html output and conversion hash
-    _noteTextHtmlConversionHash = hash;
-    _noteTextHtml = result;
-
     return result;
 }
 
@@ -1844,6 +2086,22 @@ QString Note::generateTextForLink(QString text) {
             "[^\\d\\w]", QRegularExpression::CaseInsensitiveOption |
                     QRegularExpression::UseUnicodePropertiesOption);
     text.replace(re, "_");
+
+    // if there are only numbers we also want the "@" added, because
+    // otherwise the text will get interpreted as ip address
+    QRegularExpressionMatch match =
+            QRegularExpression(R"(^(\d+)$)").match(text);
+    bool onlyNumbers = match.hasMatch();
+
+    // if the hostname of the url will get to long QUrl will not
+    // recognize it because of STD 3 rules, so we will use the
+    // username for the note name instead of the hostname
+    // the limit is 63 characters, but special characters use up
+    // far more space
+    if (text.length() > 46 || onlyNumbers) {
+        text += "@";
+    }
+
     return text;
 }
 
@@ -1974,11 +2232,16 @@ bool Note::canDecryptNoteText() {
 
     // check if a hook changed the text
     if (decryptedNoteText.isEmpty()) {
-        // decrypt the note text with Botan
-        BotanWrapper botanWrapper;
-        botanWrapper.setPassword(cryptoPassword);
-        botanWrapper.setSalt(BOTAN_SALT);
-        decryptedNoteText = botanWrapper.Decrypt(encryptedNoteText);
+        try {
+            // decrypt the note text with Botan
+            BotanWrapper botanWrapper;
+            botanWrapper.setPassword(cryptoPassword);
+            botanWrapper.setSalt(BOTAN_SALT);
+            decryptedNoteText = botanWrapper.Decrypt(encryptedNoteText);
+        }
+        catch (Botan::Exception) {
+            return false;
+        }
 
         // fallback to SimpleCrypt
         if (decryptedNoteText == "") {
@@ -2019,10 +2282,14 @@ QString Note::getDecryptedNoteText() {
     // check if a hook changed the text
     if (decryptedNoteText.isEmpty()) {
         // decrypt the note text
-        BotanWrapper botanWrapper;
-        botanWrapper.setPassword(cryptoPassword);
-        botanWrapper.setSalt(BOTAN_SALT);
-        decryptedNoteText = botanWrapper.Decrypt(encryptedNoteText);
+        try {
+            BotanWrapper botanWrapper;
+            botanWrapper.setPassword(cryptoPassword);
+            botanWrapper.setSalt(BOTAN_SALT);
+            decryptedNoteText = botanWrapper.Decrypt(encryptedNoteText);
+        }
+        catch (Botan::Exception) {
+        }
 
         // fallback to SimpleCrypt
         if (decryptedNoteText == "") {
@@ -2127,9 +2394,16 @@ bool Note::isSameFile(Note note) {
  */
 QList<int> Note::findLinkedNotes(QString fileName) {
     QString linkText = getNoteURL(fileName);
-    QList<int> noteIdList = searchInNotes("<" + linkText + ">");
-    QList<int> noteIdList2 = searchInNotes("](" + linkText + ")");
-    noteIdList.append(noteIdList2);
+    QList<int> noteIdList;
+    noteIdList.append(searchInNotes("<" + linkText + ">", true));
+    noteIdList.append(searchInNotes("](" + linkText + ")", true));
+
+    // add support for alternative links ending with "@"
+    QString altLinkText = Utils::Misc::appendIfDoesNotEndWith(linkText, "@");
+    if (altLinkText != linkText) {
+        noteIdList.append(searchInNotes("<" + altLinkText + ">", true));
+        noteIdList.append(searchInNotes("](" + altLinkText + ")", true));
+    }
 
     // remove duplicates and return list
     return noteIdList.toSet().toList();
@@ -2145,6 +2419,15 @@ const QString Note::getNoteURL(const QString &baseName) {
     return "note://" + generateTextForLink(baseName);
 }
 
+/**
+ * Returns the note-id url to a note
+ *
+ * @return
+ */
+QString Note::getNoteIdURL() {
+    return "noteid://note-" + QString::number(getId());
+}
+
 /**
  * Returns the url to a note from a file name
  *
@@ -2153,6 +2436,7 @@ const QString Note::getNoteURL(const QString &baseName) {
  */
 const QString Note::getNoteURLFromFileName(const QString &fileName) {
     QFileInfo info(fileName);
+    // TODO: baseName() will cut names like "Note 2018-07-26T18.24.22.md" down to "Note 2018-07-26T18"!
     return Note::getNoteURL(info.baseName());
 }
 
@@ -2193,6 +2477,12 @@ void Note::handleNoteRenaming(QString oldFileName, QString newFileName) {
                 QString text = note.getNoteText();
                 text.replace("<" + oldUrl + ">", "<" + newUrl + ">");
                 text.replace("](" + oldUrl + ")", "](" + newUrl + ")");
+
+                if (!oldUrl.contains("@")) {
+                    text.replace("<" + oldUrl + "@>", "<" + newUrl + ">");
+                    text.replace("](" + oldUrl + "@)", "](" + newUrl + ")");
+                }
+
                 note.storeNewText(text);
             }
     }
@@ -2205,9 +2495,9 @@ void Note::handleNoteRenaming(QString oldFileName, QString newFileName) {
  * @return
  */
 QString Note::createNoteHeader(QString name) {
-    QString header = name + "\n";
+    QString header = name.trimmed() + "\n";
 
-    for (int i = 0; i < name.length(); i++) {
+    for (int i = 0; i < min(name.length(), 40); i++) {
         header.append("=");
     }
 
@@ -2220,7 +2510,8 @@ QString Note::createNoteHeader(QString name) {
  */
 QString Note::getInsertMediaMarkdown(QFile *file, bool addNewLine,
                                      bool returnUrlOnly) {
-    if (file->exists() && (file->size() > 0)) {
+    // file->exists() is false on Arch Linux for QTemporaryFile!
+    if (file->size() > 0) {
         QDir mediaDir(NoteFolder::currentMediaPath());
 
         // created the media folder if it doesn't exist
@@ -2385,6 +2676,210 @@ bool Note::scaleDownImageFileIfNeeded(QFile &file) {
     return true;
 }
 
+/**
+ * Tries to fetch a note from an url string
+ *
+ * @param urlString
+ * @return
+ */
+Note Note::fetchByUrlString(QString urlString) {
+    QUrl url = QUrl(urlString);
+
+    // if the name of the linked note only consists of numbers we cannot use
+    // host() to get the filename, it would get converted to an ip-address
+    QRegularExpressionMatch match =
+            QRegularExpression(R"(^\w+:\/\/(\d+)$)").match(urlString);
+    QString fileName = match.hasMatch() ? match.captured(1) : url.host();
+
+    // we are using the user name as fallback if the hostname was too long
+    if (!url.userName().isEmpty()) {
+        fileName = url.userName();
+    } else {
+        if (fileName.isEmpty()) {
+            return Note();
+        }
+
+        // add a ".com" to the filename to simulate a valid domain
+        fileName += ".com";
+
+        // convert the ACE to IDN (internationalized domain names) to support
+        // links to notes with unicode characters in their names
+        // then remove the ".com" again
+        fileName = Utils::Misc::removeIfEndsWith(
+                QUrl::fromAce(fileName.toLatin1()), ".com");
+
+        // if it seem we have unicode characters in our filename let us use
+        // wildcards for each number, because full width numbers get somehow
+        // translated to normal numbers by the QTextEdit
+        if (fileName != url.host()) {
+            fileName.replace("1", "[1１]")
+                    .replace("2", "[2２]")
+                    .replace("3", "[3３]")
+                    .replace("4", "[4４]")
+                    .replace("5", "[5５]")
+                    .replace("6", "[6６]")
+                    .replace("7", "[7７]")
+                    .replace("8", "[8８]")
+                    .replace("9", "[9９]")
+                    .replace("0", "[0０]");
+        }
+    }
+
+    // this makes it possible to search for file names containing spaces
+    // instead of spaces a "-" has to be used in the note link
+    // example: note://my-note-with-spaces-in-the-name
+    fileName.replace("-", "?").replace("_", "?");
+
+    // create a regular expression to search in sqlite note table
+    QString escapedFileName = QRegularExpression::escape(fileName);
+    escapedFileName.replace("\\?", ".");
+    QRegularExpression regExp = QRegularExpression("^" + escapedFileName + "$",
+            QRegularExpression::CaseInsensitiveOption);
+
+    qDebug() << __func__ << " - 'regExp': " << regExp;
+
+    const int noteSubFolderId = NoteSubFolder::activeNoteSubFolderId();
+    Note note;
+
+    qDebug() << __func__ << " - 'noteSubFolderId': " << noteSubFolderId;
+
+    if (noteSubFolderId > 0) {
+        note = Note::fetchByName(regExp, noteSubFolderId);
+        qDebug() << __func__ << " - 'note in sub folder': " << note;
+    }
+
+    // if we haven't found a note we try searching in all note subfolders
+    if (!note.isFetched()) {
+        note = Note::fetchByName(regExp);
+        qDebug() << __func__ << " - 'note in all sub folders': " << note;
+    }
+
+    return note;
+}
+
+/**
+ * Generates the preview text of the note
+ *
+ * @return
+ */
+QString Note::getNotePreviewText(bool asHtml, int lines) {
+    QString noteText = getNoteText();
+
+    // remove Windows line breaks
+    noteText.replace(QRegularExpression("\r\n"), "\n");
+
+    // remove headlines
+    noteText.remove(QRegularExpression("^.+\n=+\n+"));
+    noteText.remove(QRegularExpression("^# .+\n+"));
+
+    // remove multiple line breaks
+    noteText.replace(QRegularExpression("\n\n+"), "\n");
+
+    const QStringList &lineList = noteText.split("\n");
+
+    if (lineList.isEmpty()) {
+        return "";
+    }
+
+    noteText = "";
+    for (int i = 0; i < min(lines, lineList.count()); i++) {
+        if (i > 0) {
+            noteText += "\n";
+        }
+
+        QString line = lineList.at(i);
+        line = line.trimmed();
+        line.truncate(80);
+
+        noteText += line;
+    }
+
+    if (asHtml) {
+        noteText = Utils::Misc::htmlspecialchars(noteText);
+        noteText.replace("\n", "<br>");
+    }
+
+    return noteText;
+}
+
+/**
+ * Generate the preview text if multiple notes are selected
+ *
+ * @param notes
+ * @return
+ */
+QString Note::generateMultipleNotesPreviewText(QList<Note> notes) {
+    QSettings settings;
+    bool darkModeColors = settings.value("darkModeColors").toBool();
+    QString oddBackgroundColor = darkModeColors ? "#444444" : "#f1f1f1";
+    QString linkColor = darkModeColors ? "#eeeeee" : "#222222";
+
+    QString previewHtml =
+        "<html><head><style>"
+            "table, body {width: 100%;}"
+            "table td {padding: 10px}"
+            "table td.odd {background-color: " + oddBackgroundColor + ";}"
+            "p {margin: 0.5em 0 0 0;}"
+            "small {font-size: 0.8em;}"
+            "h2 {margin-bottom: 0.5em;}"
+            "h2 a {text-decoration: none; color: " + linkColor + "}"
+        "</style></head>"
+        "<body>"
+        "   <table>";
+
+    int notesCount = notes.count();
+    int displayedNotesCount = notesCount > 40 ? 40 : notesCount;
+
+    bool isOdd = false;
+    for (int i = 0; i < displayedNotesCount; i++) {
+        Note note = notes[i];
+        QString oddStyle = isOdd ? " class='odd'" : "";
+        QDateTime modified = note.getFileLastModified();
+        QString noteText = note.getNotePreviewText(true, 5);
+        QString noteLink = note.getNoteIdURL();
+
+        previewHtml +=
+            "<tr><td" + oddStyle + ">"
+            "<h2><a href='" + noteLink + "'>" + note.getName() + "</a></h2>"
+            "<small>" + modified.toString() + "</small>"
+            "<p>" + noteText + "</p>"
+            "</td></tr>";
+        isOdd = !isOdd;
+    }
+
+    if (displayedNotesCount < notesCount) {
+        previewHtml += "<tr><td>" + QObject::tr("…and %n more note(s)", "",
+                notesCount - displayedNotesCount) +
+                "</td></tr>";
+    }
+
+    previewHtml +=
+        "   </table>"
+        "</body>"
+        "</html>";
+
+    return previewHtml;
+}
+
+/**
+ * Returns the parsed bookmarks of the note for the WebSocketServerService
+ *
+ * @return
+ */
+QString Note::getParsedBookmarksWebServiceJsonText() {
+    return Bookmark::bookmarksWebServiceJsonText(getParsedBookmarks());
+}
+
+/**
+ * Returns the parsed bookmarks of the note
+ *
+ * @return
+ */
+QList<Bookmark> Note::getParsedBookmarks() {
+    QString text = decryptedNoteText.isEmpty() ? noteText : decryptedNoteText;
+    return Bookmark::parseBookmarks(text);
+}
+
 /**
  * Fetches all tags of the note
  */
