@@ -5,7 +5,9 @@
 #include <QSqlQuery>
 #include <QFile>
 #include <QUrl>
+#include <QRegularExpression>
 #include "notesubfolder.h"
+#include "bookmark.h"
 
 #define NOTE_TEXT_ENCRYPTION_PRE_STRING "<!-- BEGIN ENCRYPTED TEXT --"
 #define NOTE_TEXT_ENCRYPTION_POST_STRING "-- END ENCRYPTED TEXT -->"
@@ -41,6 +43,9 @@ public:
 
     static Note fetch(int id);
 
+    static Note fetchByName(QRegularExpression regExp,
+                            int noteSubFolderId = -1);
+
     static Note fetchByFileName(QString fileName, int noteSubFolderId = -1);
 
     static Note fetchByName(QString name, int noteSubFolderId = -1);
@@ -55,8 +60,13 @@ public:
 
     static QList<Note> search(QString text);
 
-    static QList<QString> searchAsNameList(QString text,
-                                           bool searchInNameOnly = false);
+    static QList<QString> searchAsNameListInCurrentNoteSubFolder(
+            QString text, bool searchInNameOnly = false);
+
+    static QList<QString> searchAsNameList(
+            QString text, bool searchInNameOnly = false);
+
+    static QStringList fetchNoteNamesInCurrentNoteSubFolder();
 
     static QStringList fetchNoteNames();
 
@@ -84,7 +94,8 @@ public:
 
     friend QDebug operator<<(QDebug dbg, const Note &note);
 
-    void createFromFile(QFile &file, int noteSubFolderId = 0);
+    void createFromFile(QFile &file, int noteSubFolderId = 0,
+                        bool withNoteNameHook = false);
 
     static bool deleteAll();
 
@@ -179,6 +190,8 @@ public:
 
     int getNoteSubFolderId();
 
+    bool isInCurrentNoteSubFolder();
+
     QString relativeNoteFilePath(QString separator = "");
 
     QString relativeNoteSubFolderPath();
@@ -201,8 +214,8 @@ public:
 
     qint64 getFileSize();
 
-    static Note updateOrCreateFromFile(QFile &file,
-                                       NoteSubFolder noteSubFolder);
+    static Note updateOrCreateFromFile(QFile &file, NoteSubFolder noteSubFolder,
+                                           bool withNoteNameHook = false);
 
     static QList<int> fetchAllIds(int limit = -1, int offset = -1);
 
@@ -229,6 +242,34 @@ public:
 
     void generateFileNameFromName();
 
+    QString textToMarkdownHtml(QString str, QString notesPath,
+                               int maxImageWidth = 980, bool forExport = false,
+                               bool base64Images = false);
+
+    QStringList getMediaFileList();
+
+    static Note fetchByUrlString(QString urlString);
+
+    static QList<int> fetchAllIdsByNoteTextPart(QString textPart);
+
+    QStringList getAttachmentsFileList();
+
+    QString getNotePreviewText(bool asHtml = false, int lines = 3);
+
+    static QString generateMultipleNotesPreviewText(QList<Note> notes);
+
+    bool handleNoteTextFileName();
+
+    QString getNoteIdURL();
+
+    static QString cleanupFileName(QString name);
+
+    static QString extendedCleanupFileName(QString name);
+
+    QList<Bookmark> getParsedBookmarks();
+
+    QString getParsedBookmarksWebServiceJsonText();
+
 protected:
     int id;
     QString name;
@@ -246,11 +287,8 @@ protected:
     QString cryptoPassword;
     QString shareUrl;
     int shareId;
-    bool handleNoteTextFileName();
     QRegularExpression getEncryptedNoteTextRegularExpression();
     QString getEncryptedNoteText();
-    static QString cleanupFileName(QString name);
-    static QString extendedCleanupFileName(QString name);
     QString _noteTextHtml;
     QString _noteTextHtmlConversionHash;
 
