@@ -167,6 +167,11 @@ bool NoteFolder::remove() {
         qWarning() << __func__ << ": " << query.lastError();
         return false;
     } else {
+        // remove the note history settings of the note folder
+        QSettings settings;
+        settings.remove("NoteHistory-" + QString::number(this->id));
+        settings.remove("NoteHistoryCurrentIndex-" + QString::number(this->id));
+
         return true;
     }
 }
@@ -376,6 +381,13 @@ QString NoteFolder::currentRootFolderName(bool fullPath) {
                     Utils::Misc::dirSeparator())+1);
 }
 
+/**
+ * Fetches the current trash path
+ */
+QString NoteFolder::currentTrashPath() {
+    return NoteFolder::currentLocalPath() + QDir::separator() + "trash";
+}
+
 /**
  * Fetches the current media path
  */
@@ -428,7 +440,7 @@ QString NoteFolder::fixRemotePath() {
 }
 
 /**
- * Create the notesPath and the recentNoteFolders to NoteFolder objects
+ * Migrate the notesPath and the recentNoteFolders to NoteFolder objects
  */
 bool NoteFolder::migrateToNoteFolders() {
     if (countAll() > 0) {
@@ -450,8 +462,12 @@ bool NoteFolder::migrateToNoteFolders() {
         noteFolder.setOwnCloudServerId(1);
         noteFolder.suggestRemotePath();
         noteFolder.setPriority(priority++);
+        noteFolder.setShowSubfolders(settings.value(
+                "showNoteSubFolders").toBool());
         noteFolder.store();
 
+        settings.remove("showNoteSubFolders");
+
         if (noteFolder.isFetched()) {
             noteFolder.setAsCurrent();
         }
@@ -461,7 +477,7 @@ bool NoteFolder::migrateToNoteFolders() {
             settings.value("recentNoteFolders").toStringList();
 
     // create recent note folders as NoteFolder
-    if (recentNoteFolders.size() > 0) {
+    if (!recentNoteFolders.empty()) {
         Q_FOREACH(QString recentNoteFolderPath, recentNoteFolders) {
                 if (notesPath != recentNoteFolderPath) {
                     NoteFolder noteFolder;
