@@ -5,6 +5,9 @@
 #include <QDebug>
 #include <QDir>
 
+// note sub-folders that should be ignored by default
+// regular expression, separated by ";"
+#define IGNORED_NOTE_SUBFOLDERS_DEFAULT "^\\."
 
 class NoteSubFolder {
 public:
@@ -52,7 +55,8 @@ public:
 
     NoteSubFolder getParent();
 
-    static QList<NoteSubFolder> fetchAllByParentId(int parentId);
+    static QList<NoteSubFolder> fetchAllByParentId(int parentId,
+            QString sortBy = "file_last_modified DESC");
 
     static QList<int> fetchIdsRecursivelyByParentId(int parentId);
 
@@ -62,7 +66,7 @@ public:
 
     static NoteSubFolder activeNoteSubFolder();
 
-    static bool setAsActive(int tagId);
+    static bool setAsActive(int noteSubFolderId);
 
     void setAsActive();
 
