@@ -6,6 +6,8 @@
 #include <QSqlRecord>
 #include <QSqlError>
 #include <QSettings>
+#include <utils/misc.h>
+#include <services/databaseservice.h>
 
 
 Tag::Tag() {
@@ -53,7 +55,7 @@ void Tag::setPriority(int value) {
 }
 
 Tag Tag::fetch(int id) {
-    QSqlDatabase db = QSqlDatabase::database("note_folder");
+    QSqlDatabase db = DatabaseService::getNoteFolderDatabase();
     QSqlQuery query(db);
 
     Tag tag;
@@ -67,6 +69,8 @@ Tag Tag::fetch(int id) {
         tag.fillFromQuery(query);
     }
 
+    DatabaseService::closeDatabaseConnection(db, query);
+
     return tag;
 }
 
@@ -78,7 +82,7 @@ Tag Tag::fetch(int id) {
  * @return
  */
 Tag Tag::fetchByName(QString name, bool startsWith) {
-    QSqlDatabase db = QSqlDatabase::database("note_folder");
+    QSqlDatabase db = DatabaseService::getNoteFolderDatabase();
     QSqlQuery query(db);
     Tag tag;
     QString sql = "SELECT * FROM tag WHERE name " +
@@ -97,11 +101,13 @@ Tag Tag::fetchByName(QString name, bool startsWith) {
         tag.fillFromQuery(query);
     }
 
+    DatabaseService::closeDatabaseConnection(db, query);
+
     return tag;
 }
 
 Tag Tag::fetchByName(QString name, int parentId) {
-    QSqlDatabase db = QSqlDatabase::database("note_folder");
+    QSqlDatabase db = DatabaseService::getNoteFolderDatabase();
     QSqlQuery query(db);
     Tag tag;
 
@@ -116,11 +122,13 @@ Tag Tag::fetchByName(QString name, int parentId) {
         tag.fillFromQuery(query);
     }
 
+    DatabaseService::closeDatabaseConnection(db, query);
+
     return tag;
 }
 
 int Tag::countAll() {
-    QSqlDatabase db = QSqlDatabase::database("note_folder");
+    QSqlDatabase db = DatabaseService::getNoteFolderDatabase();
     QSqlQuery query(db);
 
     query.prepare("SELECT COUNT(*) AS cnt FROM tag");
@@ -128,9 +136,14 @@ int Tag::countAll() {
     if (!query.exec()) {
         qWarning() << __func__ << ": " << query.lastError();
     } else if (query.first()) {
-        return query.value("cnt").toInt();
+        int result = query.value("cnt").toInt();
+        DatabaseService::closeDatabaseConnection(db, query);
+
+        return result;
     }
 
+    DatabaseService::closeDatabaseConnection(db, query);
+
     return 0;
 }
 
@@ -138,7 +151,7 @@ int Tag::countAll() {
  * Removes the tag, their children and its note link items
  */
 bool Tag::remove() {
-    QSqlDatabase db = QSqlDatabase::database("note_folder");
+    QSqlDatabase db = DatabaseService::getNoteFolderDatabase();
     QSqlQuery query(db);
 
     // remove the tag
@@ -147,6 +160,7 @@ bool Tag::remove() {
 
     if (!query.exec()) {
         qWarning() << __func__ << ": " << query.lastError();
+        DatabaseService::closeDatabaseConnection(db, query);
         return false;
     } else {
         // remove all children tags
@@ -154,14 +168,20 @@ bool Tag::remove() {
                 tag.remove();
             }
 
+        QSqlDatabase db = DatabaseService::getNoteFolderDatabase();
+        QSqlQuery query(db);
+
         // remove the note tag links
         query.prepare("DELETE FROM noteTagLink WHERE tag_id = :id");
         query.bindValue(":id", id);
 
         if (!query.exec()) {
             qWarning() << __func__ << ": " << query.lastError();
+            DatabaseService::closeDatabaseConnection(db, query);
+
             return false;
         } else {
+            DatabaseService::closeDatabaseConnection(db, query);
             return true;
         }
     }
@@ -186,7 +206,7 @@ bool Tag::fillFromQuery(QSqlQuery query) {
 }
 
 QList<Tag> Tag::fetchAll() {
-    QSqlDatabase db = QSqlDatabase::database("note_folder");
+    QSqlDatabase db = DatabaseService::getNoteFolderDatabase();
     QSqlQuery query(db);
 
     QList<Tag> tagList;
@@ -225,11 +245,13 @@ QList<Tag> Tag::fetchAll() {
         }
     }
 
+    DatabaseService::closeDatabaseConnection(db, query);
+
     return tagList;
 }
 
-QList<Tag> Tag::fetchAllByParentId(int parentId) {
-    QSqlDatabase db = QSqlDatabase::database("note_folder");
+QList<Tag> Tag::fetchAllByParentId(int parentId, QString sortBy) {
+    QSqlDatabase db = DatabaseService::getNoteFolderDatabase();
     QSqlQuery query(db);
     QList<Tag> tagList;
 
@@ -248,7 +270,7 @@ QList<Tag> Tag::fetchAllByParentId(int parentId) {
                   "FROM tag t LEFT JOIN noteTagLink l ON t.id = l.tag_id "
                   "WHERE parent_id = :parentId "
                   "GROUP BY t.name "
-                  "ORDER BY created DESC");
+                  "ORDER BY " + sortBy);
     query.bindValue(":parentId", parentId);
 
     if (!query.exec()) {
@@ -260,6 +282,8 @@ QList<Tag> Tag::fetchAllByParentId(int parentId) {
         }
     }
 
+    DatabaseService::closeDatabaseConnection(db, query);
+
     return tagList;
 }
 
@@ -289,7 +313,7 @@ bool Tag::isTaggingShowNotesRecursively() {
 }
 
 int Tag::countAllParentId(int parentId) {
-    QSqlDatabase db = QSqlDatabase::database("note_folder");
+    QSqlDatabase db = DatabaseService::getNoteFolderDatabase();
     QSqlQuery query(db);
 
     query.prepare("SELECT COUNT(*) AS cnt FROM tag "
@@ -299,9 +323,13 @@ int Tag::countAllParentId(int parentId) {
     if (!query.exec()) {
         qWarning() << __func__ << ": " << query.lastError();
     } else if (query.first()) {
-        return query.value("cnt").toInt();
+        int result = query.value("cnt").toInt();
+        DatabaseService::closeDatabaseConnection(db, query);
+        return result;
     }
 
+    DatabaseService::closeDatabaseConnection(db, query);
+
     return 0;
 }
 
@@ -324,7 +352,7 @@ bool Tag::hasChild(int tagId) {
  * Fetches all linked tags of a note
  */
 QList<Tag> Tag::fetchAllOfNote(Note note) {
-    QSqlDatabase db = QSqlDatabase::database("note_folder");
+    QSqlDatabase db = DatabaseService::getNoteFolderDatabase();
     QSqlQuery query(db);
 
     QList<Tag> tagList;
@@ -347,14 +375,37 @@ QList<Tag> Tag::fetchAllOfNote(Note note) {
         }
     }
 
+    DatabaseService::closeDatabaseConnection(db, query);
+
     return tagList;
 }
 
+/**
+ * Fetches all linked tags of a list of notes
+ */
+QList<Tag> Tag::fetchAllOfNotes(QList<Note> notes) {
+    QList<Tag> resultTagList;
+
+    Q_FOREACH (Note note, notes) {
+            QList<Tag> tagList = Tag::fetchAllOfNote(note);
+
+            Q_FOREACH (Tag tag, tagList) {
+                    if (!resultTagList.contains(tag)) {
+                        resultTagList.append(tag);
+                    }
+            }
+    }
+
+    qSort(resultTagList);
+
+    return resultTagList;
+}
+
 /**
  * Fetches the names of all linked tags of a note
  */
 QStringList Tag::fetchAllNamesOfNote(Note note) {
-    QSqlDatabase db = QSqlDatabase::database("note_folder");
+    QSqlDatabase db = DatabaseService::getNoteFolderDatabase();
     QSqlQuery query(db);
     QStringList tagNameList;
 
@@ -375,6 +426,34 @@ QStringList Tag::fetchAllNamesOfNote(Note note) {
         }
     }
 
+    DatabaseService::closeDatabaseConnection(db, query);
+
+    return tagNameList;
+}
+
+/**
+ * Fetches the names by substring searching for the name
+ */
+QStringList Tag::searchAllNamesByName(QString name) {
+    QSqlDatabase db = DatabaseService::getNoteFolderDatabase();
+    QSqlQuery query(db);
+    QStringList tagNameList;
+
+    query.prepare("SELECT name FROM tag "
+                          "WHERE name LIKE :name "
+                          "ORDER BY priority ASC, name ASC");
+    query.bindValue(":name", "%" + name + "%");
+
+    if (!query.exec()) {
+        qWarning() << __func__ << ": " << query.lastError();
+    } else {
+        for (int r = 0; query.next(); r++) {
+            tagNameList << query.value("name").toString();
+        }
+    }
+
+    DatabaseService::closeDatabaseConnection(db, query);
+
     return tagNameList;
 }
 
@@ -395,7 +474,7 @@ Tag Tag::fetchOneOfNoteWithColor(Note note) {
  * Count all linked tags of a note
  */
 int Tag::countAllOfNote(Note note) {
-    QSqlDatabase db = QSqlDatabase::database("note_folder");
+    QSqlDatabase db = DatabaseService::getNoteFolderDatabase();
     QSqlQuery query(db);
 
     query.prepare("SELECT COUNT(*) AS cnt FROM noteTagLink "
@@ -408,9 +487,14 @@ int Tag::countAllOfNote(Note note) {
     if (!query.exec()) {
         qWarning() << __func__ << ": " << query.lastError();
     } else if (query.first()) {
-        return query.value("cnt").toInt();
+        int result = query.value("cnt").toInt();
+        DatabaseService::closeDatabaseConnection(db, query);
+
+        return result;
     }
 
+    DatabaseService::closeDatabaseConnection(db, query);
+
     return 0;
 }
 
@@ -418,7 +502,7 @@ int Tag::countAllOfNote(Note note) {
  * Checks if tag is linked to a note
  */
 bool Tag::isLinkedToNote(Note note) {
-    QSqlDatabase db = QSqlDatabase::database("note_folder");
+    QSqlDatabase db = DatabaseService::getNoteFolderDatabase();
     QSqlQuery query(db);
 
     query.prepare("SELECT COUNT(*) AS cnt FROM noteTagLink "
@@ -433,9 +517,14 @@ bool Tag::isLinkedToNote(Note note) {
     if (!query.exec()) {
         qWarning() << __func__ << ": " << query.lastError();
     } else if (query.first()) {
-        return query.value("cnt").toInt() > 0;
+        bool result = query.value("cnt").toInt() > 0;
+        DatabaseService::closeDatabaseConnection(db, query);
+
+        return result;
     }
 
+    DatabaseService::closeDatabaseConnection(db, query);
+
     return false;
 }
 
@@ -443,7 +532,7 @@ bool Tag::isLinkedToNote(Note note) {
  * Returns all tags that are linked to certain note names
  */
 QList<Tag> Tag::fetchAllWithLinkToNoteNames(QStringList noteNameList) {
-    QSqlDatabase db = QSqlDatabase::database("note_folder");
+    QSqlDatabase db = DatabaseService::getNoteFolderDatabase();
     QSqlQuery query(db);
     QList<Tag> tagList;
     QString noteIdListString = noteNameList.join("','");
@@ -469,6 +558,8 @@ QList<Tag> Tag::fetchAllWithLinkToNoteNames(QStringList noteNameList) {
         }
     }
 
+    DatabaseService::closeDatabaseConnection(db, query);
+
     return tagList;
 }
 
@@ -476,7 +567,7 @@ QList<Tag> Tag::fetchAllWithLinkToNoteNames(QStringList noteNameList) {
  * Fetches all linked note file names
  */
 QStringList Tag::fetchAllLinkedNoteFileNames(bool fromAllSubfolders) {
-    QSqlDatabase db = QSqlDatabase::database("note_folder");
+    QSqlDatabase db = DatabaseService::getNoteFolderDatabase();
     QSqlQuery query(db);
     QStringList fileNameList;
 
@@ -485,9 +576,9 @@ QStringList Tag::fetchAllLinkedNoteFileNames(bool fromAllSubfolders) {
         query.prepare("SELECT note_file_name FROM noteTagLink WHERE tag_id = :id");
     } else {
         query.prepare("SELECT note_file_name FROM noteTagLink WHERE tag_id = :id "
-                              "AND note_sub_folder_path = :noteSubFolderPath");
+                              "AND note_sub_folder_path LIKE :noteSubFolderPath");
         query.bindValue(":noteSubFolderPath",
-                    NoteSubFolder::activeNoteSubFolder().relativePath());
+                    NoteSubFolder::activeNoteSubFolder().relativePath() + "%");
     }
 
     query.bindValue(":id", this->id);
@@ -500,15 +591,47 @@ QStringList Tag::fetchAllLinkedNoteFileNames(bool fromAllSubfolders) {
         }
     }
 
+    DatabaseService::closeDatabaseConnection(db, query);
+
     return fileNameList;
 }
 
+/**
+ * Fetches all linked notes
+ */
+QList<Note> Tag::fetchAllLinkedNotes() {
+    QSqlDatabase db = DatabaseService::getNoteFolderDatabase();
+    QSqlQuery query(db);
+    QList<Note> noteList;
+
+    query.prepare("SELECT note_file_name, note_sub_folder_path FROM "
+                  "noteTagLink WHERE tag_id = :id");
+    query.bindValue(":id", this->id);
+
+    if (!query.exec()) {
+        qWarning() << __func__ << ": " << query.lastError();
+    } else {
+        for (int r = 0; query.next(); r++) {
+            QString fileName = query.value("note_file_name").toString();
+            QString noteSubFolderPath = query.value("note_sub_folder_path").toString();
+            auto noteSubFolder = NoteSubFolder::fetchByPathData(noteSubFolderPath, "/");
+            auto note = Note::fetchByName(fileName, noteSubFolder.getId());
+
+            noteList << note;
+        }
+    }
+
+    DatabaseService::closeDatabaseConnection(db, query);
+
+    return noteList;
+}
+
 /**
  * Converts backslashes to slashes in the noteTagLink table to fix
  * problems with Windows
  */
 void Tag::convertDirSeparator() {
-    QSqlDatabase db = QSqlDatabase::database("note_folder");
+    QSqlDatabase db = DatabaseService::getNoteFolderDatabase();
     QSqlQuery query(db);
 
     query.prepare("UPDATE noteTagLink SET note_sub_folder_path = replace("
@@ -517,13 +640,15 @@ void Tag::convertDirSeparator() {
     if (!query.exec()) {
         qWarning() << __func__ << ": " << query.lastError();
     }
+
+    DatabaseService::closeDatabaseConnection(db, query);
 }
 
 /**
  * Fetches all tag names
  */
 QStringList Tag::fetchAllNames() {
-    QSqlDatabase db = QSqlDatabase::database("note_folder");
+    QSqlDatabase db = DatabaseService::getNoteFolderDatabase();
     QSqlQuery query(db);
 
     QStringList nameList;
@@ -538,23 +663,31 @@ QStringList Tag::fetchAllNames() {
         }
     }
 
+    DatabaseService::closeDatabaseConnection(db, query);
+
     return nameList;
 }
 
 /**
  * Count the linked note file names
  */
-int Tag::countLinkedNoteFileNames(bool fromAllSubfolders) {
-    QSqlDatabase db = QSqlDatabase::database("note_folder");
+int Tag::countLinkedNoteFileNames(bool fromAllSubfolders, bool recursive) {
+    QSqlDatabase db = DatabaseService::getNoteFolderDatabase();
     QSqlQuery query(db);
 
     if (fromAllSubfolders) {
     query.prepare("SELECT COUNT(note_file_name) AS cnt FROM noteTagLink "
                           "WHERE tag_id = :id");
-    } else {
+    } else if (recursive) {
         query.prepare("SELECT COUNT(note_file_name) AS cnt FROM noteTagLink "
                               "WHERE tag_id = :id AND "
-                              "note_sub_folder_path = :noteSubFolderPath");
+                              "note_sub_folder_path LIKE :noteSubFolderPath");
+        query.bindValue(":noteSubFolderPath",
+                        NoteSubFolder::activeNoteSubFolder().relativePath() + "%");
+    } else {
+        query.prepare("SELECT COUNT(note_file_name) AS cnt FROM noteTagLink "
+                      "WHERE tag_id = :id AND "
+                      "note_sub_folder_path = :noteSubFolderPath");
         query.bindValue(":noteSubFolderPath",
                         NoteSubFolder::activeNoteSubFolder().relativePath());
     }
@@ -563,9 +696,14 @@ int Tag::countLinkedNoteFileNames(bool fromAllSubfolders) {
     if (!query.exec()) {
         qWarning() << __func__ << ": " << query.lastError();
     } else if (query.first()) {
-        return query.value("cnt").toInt();
+        int result = query.value("cnt").toInt();
+        DatabaseService::closeDatabaseConnection(db, query);
+
+        return result;
     }
 
+    DatabaseService::closeDatabaseConnection(db, query);
+
     return 0;
 }
 
@@ -573,7 +711,7 @@ int Tag::countLinkedNoteFileNames(bool fromAllSubfolders) {
  * Inserts or updates a Tag object in the database
  */
 bool Tag::store() {
-    QSqlDatabase db = QSqlDatabase::database("note_folder");
+    QSqlDatabase db = DatabaseService::getNoteFolderDatabase();
     QSqlQuery query(db);
     QString colorField = colorFieldName();
 
@@ -598,6 +736,8 @@ bool Tag::store() {
     if (!query.exec()) {
         // on error
         qWarning() << __func__ << ": " << query.lastError();
+
+        DatabaseService::closeDatabaseConnection(db, query);
         return false;
     } else if (this->id == 0) {
         // on insert
@@ -620,6 +760,8 @@ bool Tag::store() {
         }
     }
 
+    DatabaseService::closeDatabaseConnection(db, query);
+
     return true;
 }
 
@@ -642,7 +784,7 @@ bool Tag::linkToNote(Note note) {
         return false;
     }
 
-    QSqlDatabase db = QSqlDatabase::database("note_folder");
+    QSqlDatabase db = DatabaseService::getNoteFolderDatabase();
     QSqlQuery query(db);
     query.prepare("INSERT INTO noteTagLink (tag_id, note_file_name, "
                           "note_sub_folder_path) "
@@ -658,6 +800,9 @@ bool Tag::linkToNote(Note note) {
         // we should not show this warning, because we don't check if a
         // link to a note already exists before we try to create an other link
 //        qWarning() << __func__ << ": " << query.lastError();
+
+        DatabaseService::closeDatabaseConnection(db, query);
+
         return false;
     }
 
@@ -677,6 +822,8 @@ bool Tag::linkToNote(Note note) {
         }
     }
 
+    DatabaseService::closeDatabaseConnection(db, query);
+
     return true;
 }
 
@@ -688,7 +835,7 @@ bool Tag::removeLinkToNote(Note note) {
         return false;
     }
 
-    QSqlDatabase db = QSqlDatabase::database("note_folder");
+    QSqlDatabase db = DatabaseService::getNoteFolderDatabase();
     QSqlQuery query(db);
     query.prepare("DELETE FROM noteTagLink WHERE tag_id = :tagId AND "
                           "note_file_name = :noteFileName AND "
@@ -702,9 +849,14 @@ bool Tag::removeLinkToNote(Note note) {
     if (!query.exec()) {
         // on error
         qWarning() << __func__ << ": " << query.lastError();
+
+        DatabaseService::closeDatabaseConnection(db, query);
+
         return false;
     }
 
+    DatabaseService::closeDatabaseConnection(db, query);
+
     return true;
 }
 
@@ -712,7 +864,7 @@ bool Tag::removeLinkToNote(Note note) {
  * Removes all links to a note
  */
 bool Tag::removeAllLinksToNote(Note note) {
-    QSqlDatabase db = QSqlDatabase::database("note_folder");
+    QSqlDatabase db = DatabaseService::getNoteFolderDatabase();
     QSqlQuery query(db);
     query.prepare("DELETE FROM noteTagLink WHERE "
                           "note_file_name = :noteFileName AND "
@@ -725,9 +877,14 @@ bool Tag::removeAllLinksToNote(Note note) {
     if (!query.exec()) {
         // on error
         qWarning() << __func__ << ": " << query.lastError();
+
+        DatabaseService::closeDatabaseConnection(db, query);
+
         return false;
     }
 
+    DatabaseService::closeDatabaseConnection(db, query);
+
     return true;
 }
 
@@ -735,7 +892,7 @@ bool Tag::removeAllLinksToNote(Note note) {
  * Removes all broken note tag links
  */
 void Tag::removeBrokenLinks() {
-    QSqlDatabase db = QSqlDatabase::database("note_folder");
+    QSqlDatabase db = DatabaseService::getNoteFolderDatabase();
     QSqlQuery query(db);
 
     query.prepare("SELECT * FROM notetaglink");
@@ -758,6 +915,8 @@ void Tag::removeBrokenLinks() {
             }
         }
     }
+
+    DatabaseService::closeDatabaseConnection(db, query);
 }
 
 /**
@@ -767,7 +926,7 @@ void Tag::removeBrokenLinks() {
  * @return
  */
 bool Tag::removeNoteLinkById(int id) {
-    QSqlDatabase db = QSqlDatabase::database("note_folder");
+    QSqlDatabase db = DatabaseService::getNoteFolderDatabase();
     QSqlQuery query(db);
     query.prepare("DELETE FROM noteTagLink WHERE id = :id");
     query.bindValue(":id", id);
@@ -775,9 +934,12 @@ bool Tag::removeNoteLinkById(int id) {
     if (!query.exec()) {
         // on error
         qWarning() << __func__ << ": " << query.lastError();
+
+        DatabaseService::closeDatabaseConnection(db, query);
         return false;
     }
 
+    DatabaseService::closeDatabaseConnection(db, query);
     return true;
 }
 
@@ -786,7 +948,7 @@ bool Tag::removeNoteLinkById(int id) {
  * Renames the note file name of note links
  */
 bool Tag::renameNoteFileNamesOfLinks(QString oldFileName, QString newFileName) {
-    QSqlDatabase db = QSqlDatabase::database("note_folder");
+    QSqlDatabase db = DatabaseService::getNoteFolderDatabase();
     QSqlQuery query(db);
     query.prepare("UPDATE noteTagLink SET note_file_name = :newFileName WHERE "
                           "note_file_name = :oldFileName AND "
@@ -800,9 +962,13 @@ bool Tag::renameNoteFileNamesOfLinks(QString oldFileName, QString newFileName) {
     if (!query.exec()) {
         // on error
         qWarning() << __func__ << ": " << query.lastError();
+
+        DatabaseService::closeDatabaseConnection(db, query);
         return false;
     }
 
+    DatabaseService::closeDatabaseConnection(db, query);
+
     return true;
 }
 
@@ -881,6 +1047,14 @@ void Tag::migrateDarkColors() {
     settings.setValue("darkMode", darkMode);
 }
 
+bool Tag::operator==(const Tag &tag) const {
+    return id == tag.id;
+}
+
+bool Tag::operator<(const Tag &tag) const {
+    return name < tag.name;
+}
+
 QDebug operator<<(QDebug dbg, const Tag &tag) {
     dbg.nospace() << "Tag: <id>" << tag.id << " <name>" << tag.name <<
             " <parentId>" << tag.parentId;
