@@ -3,15 +3,19 @@
 #include <QApplication>
 #include <QtGui>
 #include <QSettings>
+#include <QStyleFactory>
 #include <QTranslator>
 #include <services/metricsservice.h>
 #include <widgets/logwidget.h>
 #include <services/databaseservice.h>
-#include <utils/misc.h>
-#include <QStyleFactory>
 #include "libraries/singleapplication/singleapplication.h"
 #include "version.h"
 #include "release.h"
+#include <utils/misc.h>
+#include <utils/schema.h>
+
+// define the base class for SingleApplication
+#define QAPPLICATION_CLASS QApplication
 
 /**
  * Macro for loading the translations
@@ -36,6 +40,10 @@
     app.installTranslator(&translator4); \
     translator5.load("../src/languages/QOwnNotes_" + locale); \
     app.installTranslator(&translator5); \
+    translator6.load("../share/QOwnNotes/languages/QOwnNotes_" + locale); \
+    app.installTranslator(&translator6); \
+    translator7.load(appPath + "/../share/QOwnNotes/languages/QOwnNotes_" + locale); \
+    app.installTranslator(&translator7); \
     translatorLocal.load("QOwnNotes_" + locale); \
     app.installTranslator(&translatorLocal);
 
@@ -61,7 +69,7 @@
  * Does the miscellaneous startup
  * If false is returned the app is supposed to quit
  */
-bool mainStartupMisc(const QStringList arguments) {
+bool mainStartupMisc(const QStringList &arguments) {
     QCommandLineParser parser;
     parser.setApplicationDescription("QOwnNotes " + QString(VERSION));
     const QCommandLineOption helpOption = parser.addHelpOption();
@@ -69,6 +77,17 @@ bool mainStartupMisc(const QStringList arguments) {
             "portable", QCoreApplication::translate("main", "Runs the "
                     "application in portable mode."));
     parser.addOption(portableOption);
+    const QCommandLineOption dumpSettingsOption(
+            "dump-settings", QCoreApplication::translate("main", "Prints out "
+                    "a dump of the settings and other information about the "
+                    "application and environment in GitHub Markdown and exits "
+                    "the application."));
+    parser.addOption(dumpSettingsOption);
+    const QCommandLineOption allowMultipleInstancesOption(
+            "allow-multiple-instances", QCoreApplication::translate("main",
+                    "Allows multiple instances of QOwnNotes to be started "
+                    "even if disallowed in the settings."));
+    parser.addOption(allowMultipleInstancesOption);
     const QCommandLineOption clearSettingsOption(
             "clear-settings", QCoreApplication::translate("main", "Clears the "
                     "settings and runs the application."));
@@ -185,6 +204,7 @@ bool mainStartupMisc(const QStringList arguments) {
         WelcomeDialog welcomeDialog;
         // exit QOwnNotes if the welcome dialog was canceled
         if (welcomeDialog.exec() != QDialog::Accepted) {
+            settings.clear();
             return false;
         }
     }
@@ -195,6 +215,12 @@ bool mainStartupMisc(const QStringList arguments) {
     // try to create note folders if they are missing
     NoteFolder::migrateToNoteFolders();
 
+    if (parser.isSet(dumpSettingsOption)) {
+        fprintf(stdout, "%s\n",
+                Utils::Misc::generateDebugInformation().toLocal8Bit().constData());
+        exit(0);
+    }
+
     return true;
 }
 
@@ -258,10 +284,16 @@ int main(int argc, char *argv[]) {
     // temporary log output until LogWidget::logMessageOutput takes over
     qInstallMessageHandler(tempLogMessageOutput);
 
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 6, 0))
+    QApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
+#endif
+    QApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
+
     QString release = RELEASE;
     bool portable = false;
     bool clearSettings = false;
     bool snap = false;
+    bool allowOnlyOneAppInstance = true;
     QStringList arguments;
     QString appNameAdd = "";
 
@@ -281,6 +313,9 @@ int main(int argc, char *argv[]) {
             portable = true;
         } else if (arg == "--clear-settings") {
             clearSettings = true;
+        } else if (arg == "--help" || arg == "--dump-settings" ||
+            arg == "-h" || arg == "--allow-multiple-instances") {
+            allowOnlyOneAppInstance = false;
         } else if (arg == "--after-update") {
             qWarning() << __func__ << " - 'arg': " << arg;
 #if not defined(Q_OS_WIN)
@@ -365,6 +400,9 @@ int main(int argc, char *argv[]) {
     }
     qDebug() << __func__ << " - 'locale': " << locale;
 
+    // setup default schema settings
+    Utils::Schema::schemaSettings = new Utils::Schema::Settings();
+
 #ifndef QT_DEBUG
     QTranslator translatorRelease;
 #endif
@@ -382,14 +420,40 @@ int main(int argc, char *argv[]) {
     QTranslator translator3;
     QTranslator translator4;
     QTranslator translator5;
+    QTranslator translator6;
+    QTranslator translator7;
     QTranslator translatorLocal;
 
-    bool allowOnlyOneAppInstance = settings.value(
-            "allowOnlyOneAppInstance", false).toBool();
+#ifdef Q_OS_MAC
+    // we don't need this on macOS
+    allowOnlyOneAppInstance = false;
+#else
+    // if allowOnlyOneAppInstance still has the default true let's ask the
+    // settings
+    if (allowOnlyOneAppInstance) {
+        allowOnlyOneAppInstance = settings.value(
+                "allowOnlyOneAppInstance", true).toBool();
+    }
+#endif
+
+    if ( allowOnlyOneAppInstance && !SingleApplication::isSupported() ) {
+        allowOnlyOneAppInstance = false;
+        settings.setValue("allowOnlyOneAppInstance", false);
+
+        qWarning() << QCoreApplication::translate(
+                "main", "Single application mode is not supported on your "
+                        "system!");
+    }
 
     // if only one app instance is allowed use SingleApplication
     if (allowOnlyOneAppInstance) {
-        SingleApplication app(argc, argv);
+        SingleApplication app(argc, argv, false, SingleApplication::Mode::User,
+                1000, []() {
+            qWarning() << QCoreApplication::translate("main",
+                       "Another instance of QOwnNotes was already started! "
+                       "You can turn off the single instance mode in the settings"
+                       " or use the parameter --allow-multiple-instances.");
+        });
         app.setProperty("release", release);
         app.setProperty("portable", portable);
         app.setProperty("singleApplication", true);
@@ -416,7 +480,11 @@ int main(int argc, char *argv[]) {
 
         // raise the main window if app was started a 2nd time in single
         // application mode
-        QObject::connect(&app, &SingleApplication::showUp, [&] {
+        QObject::connect(&app, &SingleApplication::instanceStarted, [&] {
+            qWarning() << QCoreApplication::translate("main",
+                        "A second instance of QOwnNotes was attempted to be "
+                        "started!");
+
             w.show();
             w.raise();
             w.activateWindow();
