@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2017 Patrizio Bekerle -- http://www.bekerle.com
+ * Copyright (c) 2014-2019 Patrizio Bekerle -- http://www.bekerle.com
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,10 +38,16 @@
 #include <dialogs/tododialog.h>
 #include <dialogs/settingsdialog.h>
 #include <QPrinter>
+#include <widgets/logwidget.h>
+#include <dialogs/orphanedimagesdialog.h>
+#include <dialogs/orphanedattachmentsdialog.h>
+#include <dialogs/issueassistantdialog.h>
 #include "entities/notehistory.h"
 #include "dialogs/notediffdialog.h"
 #include "services/updateservice.h"
-#include "libraries/qmarkdowntextedit/qmarkdowntextedit.h"
+#include "widgets/qownnotesmarkdowntextedit.h"
+#include <libraries/fakevim/fakevim/fakevimhandler.h>
+#include "services/websocketserverservice.h"
 
 #define SORT_ALPHABETICAL 0
 #define SORT_BY_LAST_CHANGE 1
@@ -52,6 +58,8 @@ namespace Ui {
 class MainWindow;
 }
 
+class QOwnNotesMarkdownTextEdit;
+
 // forward declaration because of "xxx does not name a type"
 class TodoDialog;
 class SettingsDialog;
@@ -63,6 +71,7 @@ class MainWindow : public QMainWindow {
 
 Q_SIGNALS:
     void currentNoteChanged(Note &note);
+    void log(LogWidget::LogType logType, QString text);
 
 public:
     enum CreateNewNoteOption {
@@ -73,6 +82,9 @@ public:
 
         // places the cursor at the end of the note
         CursorAtEnd = 0x0002,
+
+        // don't call loadNoteDirectoryList
+        DisableLoadNoteDirectoryList = 0x0004,
     };
 
     Q_DECLARE_FLAGS(CreateNewNoteOptions, CreateNewNoteOption)
@@ -83,6 +95,8 @@ public:
 
     void setCurrentNoteText(QString text);
 
+    Note getCurrentNote();
+
     void createNewNote(
             QString name, QString text,
             CreateNewNoteOptions options = CreateNewNoteOption::None);
@@ -103,7 +117,7 @@ public:
 
     void handleInsertingFromMimeData(const QMimeData *mimeData);
 
-    QMarkdownTextEdit *activeNoteTextEdit();
+    QOwnNotesMarkdownTextEdit *activeNoteTextEdit();
 
     static MainWindow *instance();
 
@@ -127,7 +141,8 @@ public:
 
     QString selectedNoteTextEditText();
 
-    void linkTagNameToCurrentNote(QString tagName);
+    void linkTagNameToCurrentNote(QString tagName,
+                                  bool linkToSelectedNotes = false);
 
     Q_INVOKABLE void reloadTagTree();
 
@@ -140,6 +155,18 @@ public:
 
     bool jumpToNoteSubFolder(int noteSubFolderId);
 
+    QString noteTextEditCurrentWord(bool withPreviousCharacters = false);
+
+    Q_INVOKABLE void focusNoteTextEdit();
+
+    QString getLogText();
+
+    void turnOnDebugLogging();
+
+    void openIssueAssistantDialog();
+
+    void insertHtml(QString html);
+
 protected:
     void closeEvent(QCloseEvent *event);
 
@@ -150,6 +177,10 @@ protected:
 public slots:
     void setCurrentNoteFromNoteId(int noteId);
 
+    void regenerateNotePreview();
+
+    void storeUpdatedNotesToDisk();
+
 private slots:
 
     void on_noteTextEdit_textChanged();
@@ -162,8 +193,6 @@ private slots:
 
     void notesWereModified(const QString &str);
 
-    void storeUpdatedNotesToDisk();
-
     void on_actionSet_ownCloud_Folder_triggered();
 
     void on_searchLineEdit_textChanged(const QString &arg1);
@@ -340,8 +369,6 @@ private slots:
 
     void onNoteTextViewResize(QSize size, QSize oldSize);
 
-    void regenerateNotePreview();
-
     void on_actionAutocomplete_triggered();
 
     void restoreDistractionFreeMode();
@@ -496,11 +523,46 @@ private slots:
 
     void on_actionCheck_for_script_updates_triggered();
 
+    void noteTextEditResize(QResizeEvent *event);
+
+    void encryptedNoteTextEditResize(QResizeEvent *event);
+
+    void on_actionShow_local_trash_triggered();
+
+    void on_encryptedNoteTextEdit_customContextMenuRequested(const QPoint &pos);
+
+    void on_actionJump_to_note_text_edit_triggered();
+
+    void allowNoteEditing();
+
+    void on_tagTreeWidget_itemDoubleClicked(QTreeWidgetItem *item, int column);
+
+    void on_noteTreeWidget_itemDoubleClicked(QTreeWidgetItem *item, int column);
+
+    void on_noteTreeWidget_itemSelectionChanged();
+
+    void on_actionManage_orphaned_attachments_triggered();
+
+    void on_noteOperationsButton_clicked();
+
+    void on_actionImport_notes_from_text_files_triggered();
+
+    void on_actionTelegram_triggered();
+
+    void on_actionCopy_headline_triggered();
+
+    void on_action_FormatTable_triggered();
+
+    void on_navigationLineEdit_textChanged(const QString &arg1);
+
+    void initWebSocketServerService();
+
 private:
     Ui::MainWindow *ui;
     QString notesPath;
     QFileSystemWatcher noteDirectoryWatcher;
     Note currentNote;
+    QString _currentNoteTextHash;
     NoteDiffDialog *noteDiffDialog;
     QSignalMapper *recentNoteFolderSignalMapper;
     QSignalMapper *storeNoteBookmarkSignalMapper;
@@ -525,6 +587,7 @@ private:
     QHash<int, NoteHistoryItem> noteBookmarks;
     QPushButton *_updateAvailableButton;
     QLabel *_noteEditLineNumberLabel;
+    QPushButton *_readOnlyButton;
     QPushButton *_leaveDistractionFreeModeButton;
     QToolBar *_formattingToolbar;
     QToolBar *_customActionToolbar;
@@ -572,12 +635,18 @@ private:
     bool _noteSubFolderDockWidgetVisible;
     ActionDialog *_actionDialog;
     TodoDialog *_todoDialog;
+    IssueAssistantDialog *_issueAssistantDialog;
+    OrphanedImagesDialog *_orphanedImagesDialog;
+    OrphanedAttachmentsDialog *_orphanedAttachmentsDialog;
     SettingsDialog *_settingsDialog;
     bool _noteExternallyRemovedCheckEnabled;
     QList<QAction *> _noteTextEditContextMenuActions;
     QList<QAction *> _noteListContextMenuActions;
     QString _notePreviewHash;
     int _gitCommitInterval;
+    bool _noteEditIsCentralWidget;
+    bool _lastNoteSelectionWasMultiple;
+    WebSocketServerService *_webSocketServerService;
 
     void createSystemTrayIcon();
 
@@ -595,8 +664,6 @@ private:
             bool updateSelectedNote = true,
             bool addPreviousNoteToHistory = true);
 
-    void focusNoteTextEdit();
-
     void removeCurrentNote();
 
     void searchInNoteTextEdit(QString str);
@@ -674,8 +741,6 @@ private:
 
     bool isValidNoteFile(QFile *file);
 
-    void insertHtml(QString html);
-
     bool isInDistractionFreeMode();
 
     void removeSelectedTags();
@@ -751,7 +816,7 @@ private:
 
     QTreeWidgetItem *findNoteInNoteTreeWidget(Note note);
 
-    void jumpToNoteOrCreateNew();
+    void jumpToNoteOrCreateNew(bool disableLoadNoteDirectoryList = false);
 
     void updateShareButton();
 
@@ -764,7 +829,7 @@ private:
 
     void copySelectedNotesToNoteSubFolder(NoteSubFolder noteSubFolder);
 
-    void createNewNote(QString noteName = "");
+    void createNewNote(QString noteName = "", bool withNameAppend = true);
 
     bool selectedNotesHaveTags();
 
@@ -846,8 +911,6 @@ private:
                                     bool doRemove = false,
                                     bool triggerPostMethods = true);
 
-    void handleScriptingRemoveNoteTag(Note note, QString tagName);
-
     void handleScriptingNotesTagUpdating();
 
     void handleScriptingNotesTagRenaming(QString oldTagName, QString newTagName);
@@ -858,11 +921,30 @@ private:
     void directoryWatcherWorkaround(bool isNotesDirectoryWasModifiedDisabled,
                                     bool alsoHandleNotesWereModified = false);
 
-    bool allowNoteEditing() const;
-
     void setMenuEnabled(QMenu* menu, bool enabled);
 
     bool undoFormatting(QString formatter);
 
     void applyFormatter(QString formatter);
+
+    bool isNoteTextSelected();
+
+    void noteTextEditCustomContextMenuRequested(
+            QOwnNotesMarkdownTextEdit *noteTextEdit, const QPoint &pos);
+
+    void updateNoteTextEditReadOnly();
+
+    int getSelectedNotesCount() const;
+
+    void updateNoteTreeWidgetItem(Note &note,
+                                  QTreeWidgetItem *noteItem = nullptr);
+
+    void initFakeVim(QOwnNotesMarkdownTextEdit *noteTextEdit);
+
+    void openNotesContextMenu(const QPoint &globalPos,
+            bool multiNoteMenuEntriesOnly = false);
+
+    void updateCurrentNoteTextHash();
+
+    void centerAndResize();
 };
