@@ -22,6 +22,9 @@
 #include "libraries/versionnumber/versionnumber.h"
 #include "entities/calendaritem.h"
 #include "cryptoservice.h"
+#include <QRegularExpression>
+#include <QRegularExpressionMatch>
+#include <utils/gui.h>
 
 const QString OwnCloudService::rootPath =
         "/index.php/apps/qownnotesapi/api/v1/";
@@ -179,9 +182,11 @@ void OwnCloudService::slotCalendarAuthenticationRequired(
     Q_UNUSED(authenticator);
     qWarning() << "Calendar username and/or password incorrect";
 
-    QMessageBox::warning(
-            0, tr("Username / password error"),
-            tr("Your calendar username or password is incorrect!"));
+    if (!Utils::Gui::isMessageBoxPresent()) {
+        QMessageBox::warning(
+                0, tr("Username / password error"),
+                tr("Your calendar username or password is incorrect!"));
+    }
 
     reply->abort();
 }
@@ -496,7 +501,7 @@ void OwnCloudService::checkAppVersion(QNetworkReply *reply) {
     if (serverAppVersion < minAppVersion) {
         MainWindow *mainWindow = MainWindow::instance();
 
-        if (mainWindow == NULL) {
+        if (mainWindow == nullptr) {
             return;
         }
 
@@ -794,7 +799,6 @@ void OwnCloudService::fetchShares(QString path) {
 void OwnCloudService::removeCalendarItem(CalendarItem calItem,
                                          TodoDialog *dialog) {
     this->todoDialog = dialog;
-    this->calendarName = calendarName;
 
     QUrl url(calItem.getUrl());
     QNetworkRequest r(url);
@@ -964,13 +968,16 @@ bool OwnCloudService::hasOwnCloudSettings(bool withEnabledCheck) {
  */
 void OwnCloudService::showOwnCloudServerErrorMessage(
         QString message, bool withSettingsButton) {
-    QString headline = tr("ownCloud server connection error");
+    QString headline = Utils::Misc::replaceOwnCloudText(
+            tr("ownCloud server connection error"));
     QString text = message.isEmpty() ?
             "Cannot connect to your ownCloud server! "
             "Please check your ownCloud configuration." :
             tr("ownCloud server error: <strong>%1</strong><br />"
             "Please check your ownCloud configuration.").arg(message);
 
+    text = Utils::Misc::replaceOwnCloudText(text);
+
     showOwnCloudMessage(headline, text, withSettingsButton);
 }
 
@@ -987,6 +994,13 @@ void OwnCloudService::showOwnCloudMessage(
         message = tr("You need to setup your ownCloud server!");
     }
 
+    // don't show an actual message box if there already is one present on
+    // the screen
+    if (Utils::Gui::isMessageBoxPresent()) {
+        qWarning() << headline << ": " << message;
+        return;
+    }
+
     if (withSettingsButton) {
         if (QMessageBox::warning(
                 0, headline, message,
@@ -995,7 +1009,7 @@ void OwnCloudService::showOwnCloudMessage(
 #ifndef INTEGRATION_TESTS
             MainWindow *mainWindow = MainWindow::instance();
 
-            if (mainWindow != NULL) {
+            if (mainWindow != nullptr) {
                 mainWindow->openSettingsDialog(SettingsDialog::OwnCloudPage);
             }
 #endif
@@ -1009,11 +1023,11 @@ void OwnCloudService::showOwnCloudMessage(
  * Returns the global OwnCloudService instance
  */
 OwnCloudService *OwnCloudService::instance() {
-    OwnCloudService *instance =
+    auto *instance =
             qApp->property("ownCloudService").value<OwnCloudService *>();
 
-    if (instance == NULL) {
-        instance = new OwnCloudService(NULL);
+    if (instance == nullptr) {
+        instance = new OwnCloudService(nullptr);
 
         qApp->setProperty("ownCloudService",
                           QVariant::fromValue<OwnCloudService *>(instance));
