@@ -419,15 +419,29 @@ QVariant ScriptingService::callNoteTaggingHook(Note note, QString action,
  * @return true if a function was found
  */
 bool ScriptingService::noteTaggingHookExists() {
+    return methodExists("noteTaggingHook(QVariant,QVariant,QVariant,QVariant)");
+}
+
+/**
+ * Checks if a handleNoteNameHook function exists in a script
+ * @return true if a function was found
+ */
+bool ScriptingService::handleNoteNameHookExists() {
+    return methodExists("handleNoteNameHook(QVariant)");
+}
+
+/**
+ * Checks if a method exists in a script
+ * @return true if method was found
+ */
+bool ScriptingService::methodExists(QString methodName) {
     QMapIterator<int, ScriptComponent> i(_scriptComponents);
 
     while (i.hasNext()) {
         i.next();
         ScriptComponent scriptComponent = i.value();
 
-        if (methodExistsForObject(
-                scriptComponent.object,
-                "noteTaggingHook(QVariant,QVariant,QVariant,QVariant)")) {
+        if (methodExistsForObject(scriptComponent.object, methodName)) {
             return true;
         }
     }
@@ -435,6 +449,38 @@ bool ScriptingService::noteTaggingHookExists() {
     return false;
 }
 
+/**
+ * Calls the autocompletionHook function for all script components
+ * This function is called when autocompletion is invoked in a note
+ *
+ * @return QStringList of text for the autocomplete list
+ */
+QStringList ScriptingService::callAutocompletionHook() {
+    QMapIterator<int, ScriptComponent> i(_scriptComponents);
+    QStringList results;
+
+    while (i.hasNext()) {
+        i.next();
+        ScriptComponent scriptComponent = i.value();
+        QVariant result;
+
+        if (methodExistsForObject(
+                scriptComponent.object,
+                "autocompletionHook()")) {
+            QMetaObject::invokeMethod(scriptComponent.object,
+                                      "autocompletionHook",
+                                      Q_RETURN_ARG(QVariant, result));
+
+            if (!result.isNull()) {
+                results.append(result.toStringList());
+            }
+        }
+    }
+
+    results.sort();
+    return results;
+}
+
 /**
  * Calls the insertingFromMimeDataHook function for an object
  */
@@ -547,7 +593,7 @@ QString ScriptingService::callHandleNewNoteHeadlineHookForObject(
 }
 
 /**
- * Calls the handleNoteOpenedHook function for all script components
+ * Calls the noteOpenedHook function for all script components
  */
 void ScriptingService::callHandleNoteOpenedHook(Note *note) {
     QMapIterator<int, ScriptComponent> i(_scriptComponents);
@@ -568,6 +614,33 @@ void ScriptingService::callHandleNoteOpenedHook(Note *note) {
     }
 }
 
+/**
+ * Calls the handleNoteNameHook function for all script components
+ */
+QString ScriptingService::callHandleNoteNameHook(Note *note) {
+    QMapIterator<int, ScriptComponent> i(_scriptComponents);
+
+    while (i.hasNext()) {
+        i.next();
+        ScriptComponent scriptComponent = i.value();
+        QObject *object = scriptComponent.object;
+
+        if (methodExistsForObject(object, "handleNoteNameHook(QVariant)")) {
+            NoteApi *noteApi = new NoteApi();
+            noteApi->fetch(note->getId());
+
+            QVariant text;
+            QMetaObject::invokeMethod(object, "handleNoteNameHook",
+                                      Q_RETURN_ARG(QVariant, text),
+                                      Q_ARG(QVariant, QVariant::fromValue(
+                                              static_cast<QObject*>(noteApi))));
+            return text.toString();
+        }
+    }
+
+    return "";
+}
+
 /**
  * Calls the handleNewNoteHeadlineHook function for all script components
  */
@@ -689,6 +762,27 @@ QString ScriptingService::callEncryptionHook(QString text, QString password,
     return "";
 }
 
+/**
+ * Calls the noteDoubleClickedHook function for all script components
+ */
+void ScriptingService::callHandleNoteDoubleClickedHook(Note *note) {
+    QMapIterator<int, ScriptComponent> i(_scriptComponents);
+
+    while (i.hasNext()) {
+        i.next();
+        ScriptComponent scriptComponent = i.value();
+        QObject *object = scriptComponent.object;
+
+        if (methodExistsForObject(object, "noteDoubleClickedHook(QVariant)")) {
+            NoteApi *noteApi = new NoteApi();
+            noteApi->fetch(note->getId());
+
+            QMetaObject::invokeMethod(object, "noteDoubleClickedHook",
+                                      Q_ARG(QVariant, QVariant::fromValue(
+                                              static_cast<QObject*>(noteApi))));
+        }
+    }
+}
 /**
  * QML wrapper to start a detached process
  *
@@ -812,6 +906,43 @@ QString ScriptingService::noteTextEditSelectedText() {
 #endif
 }
 
+/**
+ * Selects all text in the note text edit
+ */
+void ScriptingService::noteTextEditSelectAll() {
+    MetricsService::instance()->sendVisitIfEnabled(
+            "scripting/" + QString(__func__));
+
+#ifndef INTEGRATION_TESTS
+    MainWindow *mainWindow = MainWindow::instance();
+    if (mainWindow != Q_NULLPTR) {
+        mainWindow->activeNoteTextEdit()->selectAll();
+    }
+#endif
+}
+
+/**
+ * Reads the current word in the note text edit
+ *
+ * @param withPreviousCharacters also get more characters at the beginning
+ *                               to get characters like "@" that are not
+ *                               word-characters
+ * @return
+ */
+QString ScriptingService::noteTextEditCurrentWord(bool withPreviousCharacters) {
+    MetricsService::instance()->sendVisitIfEnabled(
+            "scripting/" + QString(__func__));
+
+#ifndef INTEGRATION_TESTS
+    MainWindow *mainWindow = MainWindow::instance();
+    return mainWindow != Q_NULLPTR ?
+           mainWindow->noteTextEditCurrentWord(withPreviousCharacters) : "";
+#else
+    Q_UNUSED(withPreviousCharacters);
+    return "";
+#endif
+}
+
 /**
  * Tags the current note with a tag named tagName
  *
@@ -840,7 +971,15 @@ void ScriptingService::log(QString text) {
     MetricsService::instance()->sendVisitIfEnabled(
             "scripting/" + QString(__func__));
 
-    LogWidget::instance()->log(LogWidget::ScriptingLogType, text);
+#ifndef INTEGRATION_TESTS
+    MainWindow *mainWindow = MainWindow::instance();
+
+    if (mainWindow != Q_NULLPTR) {
+        emit(mainWindow->log(LogWidget::ScriptingLogType, text));
+    }
+#else
+    Q_UNUSED(text);
+#endif
 }
 
 /**
@@ -862,7 +1001,7 @@ QString ScriptingService::downloadUrlToString(QUrl url) {
  *
  * @param {QString} url
  * @param {bool} returnUrlOnly if true only the media url will be returned (default false)
- * @return {QString} the media url
+ * @return {QString} the media markdown or url
  */
 QString ScriptingService::downloadUrlToMedia(QUrl url, bool returnUrlOnly) {
     MetricsService::instance()->sendVisitIfEnabled(
@@ -871,6 +1010,44 @@ QString ScriptingService::downloadUrlToMedia(QUrl url, bool returnUrlOnly) {
     return Note::downloadUrlToMedia(url, returnUrlOnly);
 }
 
+/**
+ * QML wrapper to insert a media file into the media folder and returning
+ * the media url or the markdown image text of the media
+ *
+ * @param {QString} mediaFilePath
+ * @param {bool} returnUrlOnly if true only the media url will be returned (default false)
+ * @return {QString} the media markdown or url
+ */
+QString ScriptingService::insertMediaFile(QString mediaFilePath,
+                                          bool returnUrlOnly) {
+    MetricsService::instance()->sendVisitIfEnabled(
+            "scripting/" + QString(__func__));
+
+    QFile *mediaFile = new QFile(mediaFilePath);
+
+    if (!mediaFile->exists()) {
+        return "";
+    }
+
+    return Note::getInsertMediaMarkdown(mediaFile, true, returnUrlOnly);
+}
+
+/**
+ * Regenerates the note preview
+ */
+void ScriptingService::regenerateNotePreview() {
+#ifndef INTEGRATION_TESTS
+    MainWindow *mainWindow = MainWindow::instance();
+
+    if (mainWindow != Q_NULLPTR) {
+        MetricsService::instance()->sendVisitIfEnabled(
+                "scripting/" + QString(__func__));
+
+        mainWindow->regenerateNotePreview();
+    }
+#endif
+}
+
 /**
  * Registers a custom action
  *
@@ -1096,6 +1273,21 @@ NoteApi* ScriptingService::fetchNoteByFileName(QString fileName,
     return NoteApi::fromNote(Note::fetchByFileName(fileName, noteSubFolderId));
 }
 
+/**
+ * Fetches a note by its id
+ *
+ * @param id int the id of the note
+ * @return NoteApi*
+ */
+NoteApi* ScriptingService::fetchNoteById(int id) {
+    MetricsService::instance()->sendVisitIfEnabled(
+            "scripting/" + QString(__func__));
+
+    NoteApi *note = new NoteApi();
+    note->fetch(id);
+    return note;
+}
+
 /**
  * Checks if a note file exists by its file name
  *
@@ -1181,6 +1373,41 @@ void ScriptingService::informationMessageBox(QString text, QString title) {
 #endif
 }
 
+/**
+ * Shows a question message box
+ *
+ * For information about buttons see:
+ * https://doc.qt.io/qt-5/qmessagebox.html#StandardButton-enum
+ *
+ * @param text
+ * @param title (optional)
+ * @param buttons buttons that should be shown (optional)
+ * @param defaultButton default button that will be selected (optional)
+ * @return id of pressed button
+ */
+int ScriptingService::questionMessageBox(
+        QString text, QString title, int buttons, int defaultButton) {
+    MetricsService::instance()->sendVisitIfEnabled(
+            "scripting/" + QString(__func__));
+
+#ifndef INTEGRATION_TESTS
+    MainWindow *mainWindow = MainWindow::instance();
+    if (mainWindow != Q_NULLPTR) {
+        return QMessageBox::question(mainWindow, title, text,
+                                     QMessageBox::StandardButtons(buttons),
+                                     QMessageBox::StandardButton(
+                                             defaultButton));
+    }
+#else
+    Q_UNUSED(text);
+    Q_UNUSED(title);
+    Q_UNUSED(buttons);
+    Q_UNUSED(defaultButton);
+#endif
+
+    return QMessageBox::NoButton;
+}
+
 /**
  * Shows an open file dialog
  *
@@ -1208,6 +1435,33 @@ QString ScriptingService::getOpenFileName(QString caption, QString dir,
     return "";
 }
 
+/**
+ * Shows an save file dialog
+ *
+ * @param caption (optional)
+ * @param dir (optional)
+ * @param filter (optional)
+ * @return QString
+ */
+QString ScriptingService::getSaveFileName(QString caption, QString dir,
+                                          QString filter) {
+    MetricsService::instance()->sendVisitIfEnabled(
+            "scripting/" + QString(__func__));
+
+#ifndef INTEGRATION_TESTS
+    MainWindow *mainWindow = MainWindow::instance();
+    if (mainWindow != Q_NULLPTR) {
+        return QFileDialog::getSaveFileName(mainWindow, caption, dir, filter);
+    }
+#else
+    Q_UNUSED(caption);
+    Q_UNUSED(dir);
+    Q_UNUSED(filter);
+#endif
+
+    return "";
+}
+
 /**
  * Returns path with the '/' separators converted to separators that are
  * appropriate for the underlying operating system.
@@ -1270,6 +1524,45 @@ QStringList ScriptingService::selectedNotesPaths() {
     return selectedNotePaths;
 }
 
+/**
+ * Returns a list of the ids of all selected notes
+ *
+ * Unfortunately there is no easy way to use a QList<NoteApi*> in QML, so we
+ * only will transfer the note ids
+ *
+ * @return {QList<int>} list of selected note ids
+ */
+QList<int> ScriptingService::selectedNotesIds() {
+    QList<int> selectedNotesIds;
+    MetricsService::instance()->sendVisitIfEnabled(
+            "scripting/" + QString(__func__));
+
+#ifndef INTEGRATION_TESTS
+    MainWindow *mainWindow = MainWindow::instance();
+
+    if (mainWindow != Q_NULLPTR) {
+        Q_FOREACH(Note note, mainWindow->selectedNotes()) {
+                selectedNotesIds << note.getId();
+            }
+    }
+#endif
+
+    return selectedNotesIds;
+}
+
+/**
+ * Returns a list of note ids of all notes with a certain text in the note text
+ *
+ * Unfortunately there is no easy way to use a QList<NoteApi*> in QML, so we
+ * can only transfer the note ids
+ *
+ * @return {QList<int>} list of note ids
+ */
+QList<int> ScriptingService::fetchNoteIdsByNoteTextPart(QString text) {
+    QList<int> noteIds = Note::fetchAllIdsByNoteTextPart(text);
+    return noteIds;
+}
+
 /**
  * Opens an input dialog with a select box
  *
@@ -1304,7 +1597,7 @@ QString ScriptingService::inputDialogGetItem(
  *
  * @param title {QString} title of the dialog
  * @param label {QString} label text of the dialog
- * @param text {QString} text in the dialog
+ * @param text {QString} text in the dialog (optional)
  * @return
  */
 QString ScriptingService::inputDialogGetText(
@@ -1406,3 +1699,87 @@ bool ScriptingService::jumpToNoteSubFolder(const QString &noteSubFolderPath,
     return false;
 #endif
 }
+
+/**
+ * Fetches all tags by doing a substring search on the name field
+ *
+ * @param name {QString} name to search for
+ * @return {QStringList} list of tag names
+ */
+QStringList ScriptingService::searchTagsByName(QString name) {
+    MetricsService::instance()->sendVisitIfEnabled(
+            "scripting/" + QString(__func__));
+
+    QStringList tags = Tag::searchAllNamesByName(name);
+    return tags;
+}
+
+/**
+ * Writes a text to a file
+ *
+ * @param filePath
+ * @param data
+ * @return
+ */
+bool ScriptingService::writeToFile(const QString &filePath, const QString &data)
+{
+    if (filePath.isEmpty())
+        return false;
+
+    QFile file(filePath);
+    if (!file.open(QFile::WriteOnly | QFile::Truncate))
+        return false;
+
+    QTextStream out(&file);
+    out.setCodec("UTF-8");
+    out << data;
+    file.close();
+    return true;
+}
+
+/**
+ * Triggers a menu action
+ *
+ * @param objectName {QString} object name of the action to trigger
+ * @param checked {QString} only trigger the action if checked-state is
+ *                          different than this parameter (can be 0 or 1)
+ */
+void ScriptingService::triggerMenuAction(QString objectName, QString checked) {
+    MetricsService::instance()->sendVisitIfEnabled(
+            "scripting/" + QString(__func__));
+
+#ifndef INTEGRATION_TESTS
+    MainWindow *mainWindow = MainWindow::instance();
+    if (mainWindow == Q_NULLPTR) {
+        return;
+    }
+
+    QAction *action = mainWindow->findAction(objectName);
+
+    // return if action wasn't found
+    if (action == Q_NULLPTR) {
+        return;
+    }
+
+    // script wants to set a checked state
+    if (checked != "") {
+        // return if action is not checkable
+        if (!action->isCheckable()) {
+            return;
+        }
+
+        bool isChecked = checked == "1";
+
+        // return if action already had the desired checked state
+        if (isChecked && action->isChecked()) {
+            return;
+        }
+    }
+
+    // trigger the action
+    action->trigger();
+#else
+    Q_UNUSED(objectName);
+    Q_UNUSED(checked);
+#endif
+}
